

/* Program cac11 */
/* Changing system components */

#include "cacint.h"

int main()
{
  LI noerr, iSiO2;
  char scname[TQSTRLEN], newsc[4][TQSTRLEN];
  DB wmass, stoi[3];
  

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open the thermochemical data-file cosi.dat (system C-O-Si)
     for reading */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);

  /* Set amount unit to gram, so we get the molecular mass in g/mol */
  tqcsu("Amount", "gram", &noerr);

  /* Get the name of the first system component */
  tqgnsc(1, scname, &noerr);

  /* Get its molecular mass */
  tqstsc(1, stoi, &wmass, &noerr);

  printf("Name of first system component: %s\n", scname);
  printf("Molecular mass in g/mol: %g\n", wmass);


  /* Get the stoichiometry matrix of SiO2(quartz) */
  tqinp("SiO2(quartz)",&iSiO2,&noerr);
  tqstpc(iSiO2,1,stoi,&wmass,&noerr);

  /* Show the stoichiometry matrix of SiO2(quartz) */
  printf("Stoichiometry matrix of SiO2 "
	 "in terms of system components C-O-Si:\n");
  printf("%6.3f  %6.3f  %6.3f\n", 
	 stoi[0], stoi[1], stoi[2]); 


  /* Change system components from C-O-Si to SiO-Si2C-CO2. Note that
     SiO-Si2C-CO2 form a system of three linearly independent
     components, just as the three elements C-O-Si do. */

  /* Note that when using C/C++ in contrast to FORTRAN (see above),
     the array of chars which is passed to tqcsc ("newsc" below) has
     to consist of one more string than there are system components,
     and this one extra string has to be empty. This is needed by
     ChemApp's C/C++ Interface to properly pass arrays of chars to
     FORTRAN */
  strcpy(newsc[0],"SiO");
  strcpy(newsc[1],"Si2C");
  strcpy(newsc[2],"CO2");
  strcpy(newsc[3],"");

  tqcsc((CHP)newsc,&noerr); 
  /* Note that an explicit typecast has been used in the above call to
     tqcsc to avoid potential compiler warnings about newsc being an
     incompatible pointer type */

  /* Now get the name and molecular mass of the first system 
     component again */
  printf("After changing the system components:\n");

  /* Get the name of the first system component */
  tqgnsc(1, scname, &noerr);

  /* Get its molecular mass */
  tqstsc(1, stoi, &wmass, &noerr);

  printf("Name of first system component: %s\n", scname);
  printf("Molecular mass in g/mol: %g\n", wmass);


  /* Get and display the new stoichiometry matrix for SiO2(quartz): */
  tqstpc(iSiO2,1,stoi,&wmass,&noerr);
  printf("Stoichiometry matrix of SiO2 "
	 "in terms of system components SiO-Si2C-CO2:\n");
  printf("%6.3f  %6.3f  %6.3f\n", 
	 stoi[0], stoi[1], stoi[2]);


  return 0;
 
}


