

/* Program cac14 */
/* Changing the Status of Phases */

#include "cacint.h"

void table()
{

  LI i, noerr, nphase;
  DB amount, act;
  char name[TQSTRLEN];

  /* Retrieve and display the activities and equilibrium 
     amounts of all phases */

  /* Get number of phases */
  tqnop(&nphase, &noerr);

  /* Print table header */
  printf("%-27s%-16s%-16s\n", "Phase name", "amount/mol", "activity");

  for(i = 1; i <= nphase; i++) {

    /* Get the phase name */
    tqgnp(i, name, &noerr);

    /* Get its equilibrium amount */
    tqgetr("a",  i, 0, &amount, &noerr);

    /*  Get its activity */
    tqgetr("ac",  i, 0, &act, &noerr);
    
    printf("%-24s%14.5f%14.5f\n", name, amount, act);

  }
}


int main()
{

  LI noerr, numcon;
  LI iSiC, iSiO2, iC;
  DB vals[2];
  char status[TQSTRLEN];
 
  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);

  /* Get the phase index number of SiC */
  tqinp("SiC", &iSiC, &noerr);

  /* Get the status of this phase */
  tqgsp(iSiC, status, &noerr);

  printf("The status of SiC is currently %s\n", status);

    
  /* Unless a phase is marked "dormant" or "eliminated" in the data-file
     itself, the status is by default always "entered". This means it is
     considered in the subsequent equilibrium calculations and is included
     in the mass balance. */

  /* Set some conditions and perform a simple calculation to 
     demonstrate the effect of the different statuses */

  /* Set temperature to 1900 K */
  tqsetc("T", 0, 0, 1900.0, &numcon, &noerr);

  /* Set incoming amount to 1 mol SiO2(quartz) and 2 mol C */
  tqinp("SiO2(quartz)", &iSiO2, &noerr);
  tqsetc("ia", iSiO2, 0, 1.0, &numcon, &noerr);
    
  tqinp("C", &iC, &noerr);
  tqsetc("ia", iC, 0, 2.0, &numcon, &noerr);

  /* Calculate the equilibrium */
  tqce(" ", 0, 0, vals, &noerr);

  /* Print a table containing the phase names, their
     equilibrium amounts, and activities */
  table();


  /* From the above table it can be seen that SiC is stable
     under those conditions. Its activity is unity and its 
     equilibrium amount is approx. 0.65 mol */

  /* Now change the status of SiC to "dormant" and rerun the 
     calculation */

  tqcsp(iSiC, "dormant", &noerr);
  tqce(" ", 0, 0, vals, &noerr);
  table();

  
  /* Setting a phase to "dormant" tells ChemApp to exclude it from the
     mass balance, but still calculate its activity.  This offers the
     possibility to consider a phase to be metastable, while still
     obtaining information about its activity. It can be seen from
     the above table that ChemApp sets the equilibrium amount of SiC
     to zero, and calculated its activity to be almost 30. Instead of
     SiC, ChemApp now calculates Si to be stable. */

  /* If a phase is to be completely ignored during the equilibrium
     calculations, its status should be set to "eliminated", as in the
     following example. */

  tqcsp(iSiC, "eliminated", &noerr);
  tqce(" ", 0, 0, vals, &noerr);
  table();

    
  /* Now ChemApp ignored SiC during the equilibrium calculations, as
     if it were not even included in the data-file. The status
     "eliminated" allows one to selectively delete phases from
     equilibrium calculations, for instance to increase computational
     efficiency. */

  return 0;
}

