

/* Program cac15 */
/* Changing the status of phase constituents*/

#include "cacint.h"

void table()
{

  LI i, noerr, npcon;
  DB amount, fug;
  char name[TQSTRLEN], status[TQSTRLEN];

  /* Retrieve and display the fugacities, equilibrium 
     amounts, and statuses of all phase constituents in the gas phase */

  /* Get number of phase constituents */
  tqnopc(1, &npcon, &noerr);

  /* Print table header */
  printf("%-24s  %-12s  %-12s%s\n", "Phase const. name", 
	 "amount/mol", "fugacity", "status");

  for(i = 1; i <= npcon; i++) {

    /* Get the phase constituent name */
    tqgnpc(1, i, name, &noerr);

    /* Get its equilibrium amount */
    tqgetr("a", 1, i, &amount, &noerr);

    /* Get its fugacity */
    tqgetr("ac",  1, i, &fug, &noerr);
    
    /* Get its status */
    tqgspc(1, i, status, &noerr);

    printf("%-24s  %10.5g  %10.5g   %-6s\n", 
	   name, amount, fug, status);

  }
}

int main()
{
  LI i, noerr, numcon, npcon;
  LI iSiO2, iC, iSi;
  DB vals[2], stoi[3], wmass;
 
  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);

  /* Unless a phase constituent is marked "dormant" or "eliminated" in
     the data-file itself, the status is by default always
     "entered". This means it is considered in the subsequent
     equilibrium calculation and is included in the mass balances. */

  /* Set some conditions and perform a simple calculation to 
     demonstrate the effect of the different statuses */

  /* Set temperature to 1900 K */
  tqsetc("T", 0, 0, 1900.0, &numcon, &noerr);

  /* Set incoming amount to 1 mol SiO2(quartz) and 2 mol C */
  tqinp("SiO2(quartz)", &iSiO2, &noerr);
  tqsetc("ia", iSiO2, 0, 1.0, &numcon, &noerr);
    
  tqinp("C", &iC, &noerr);
  tqsetc("ia", iC, 0, 2.0, &numcon, &noerr);

  /* Calculate the equilibrium */
  tqce(" ", 0, 0, vals, &noerr);

  /* Print a table containing the phase constituent names, their
     equilibrium amounts, activities, and status */
  table();



  /* To demonstrate the effect of assigning a different status to phase
     constituents, set the status of all those phase constituents of the
     gas phase to "dormant" which contain Si. */

  /* Get number of gas phase constituents */
  tqnopc(1, &npcon, &noerr);

  /*  Get the index number of system component Si */
  tqinsc("Si", &iSi, &noerr);

  /* Use tqstpc to determine for each phase constituent whether it
     contains Si, and set its status to "dormant" if it does */
  for (i = 1; i <= npcon; i++) {
    tqstpc(1, i, stoi, &wmass, &noerr);
    if (stoi[iSi-1] > 0.0) {
      tqcspc(1, i, "dormant", &noerr);
    }
  }

  /* Calculate the equilibrium and print the table again */
  tqce(" ", 0, 0, vals, &noerr);
  table();


  /* As can be seen from the table above, all phase constituents of the gas
     phase that contain Si have been set to "dormant". ChemApp considers
     them in the activity calculation, but does not include them in the
     mass balances: their equilibrium amount is always zero. Since this
     forces ChemApp to keep the gas phase free of Si compounds, the 1 mol
     of Si input into the system can thus only be found in the condensed
     phases. */

  /* Setting the status of these phase constituents to "eliminated"
     instead would cause ChemApp to exclude them completely from the
     equilibrium calculation. */

  return 0;
 
}


