

/* Program cac16 */
/* Set equilibrium conditions */

#include "cacint.h"

int main()
{
  LI noerr, numcon, iSiO2, iC, iSiC, iSi;
  DB vals[2], result;

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);

  /* Set conditions for a simple equilibrium calculation: temperature and
     some incoming amounts. */
   
  /* Set temperature to 1900 K */
  tqsetc("T", 0, 0, 1900.0, &numcon, &noerr);

  /* Set incoming amounts: 1 mol SiO2(quartz), 2 mol C, and 3 mol SiC */

  /* Get phase index number for both phases and set their amounts */
  tqinp("SiO2(quartz)", &iSiO2, &noerr);
  tqsetc("ia", iSiO2, 0, 1.0, &numcon, &noerr);

  tqinp("C", &iC, &noerr);
  tqsetc("ia", iC, 0, 2.0, &numcon, &noerr);

  tqinp("SiC", &iSiC, &noerr);
  tqsetc("ia", iSiC, 0, 3.0, &numcon, &noerr);

  /* Use tqshow to check which conditions have been set so far, or are
     active by default */
  tqshow(&noerr);


  /* tqsetc returns a variable (called numcon in this example) which can be
     used to remove a previously set condition. To do this, pass it to
     tqremc. Here we remove the last condition set, which are the 3 mol of
     SiC */
  tqremc(numcon, &noerr);

  /* Call tqshow again to demonstrate that it has been deleted from the 
     list of active conditions */
  tqshow(&noerr);


  /* Calculate the equilibrium */
  tqce(" ", 0, 0, vals, &noerr);

  /* Get the activity of Si */
  tqinp("Si", &iSi, &noerr);
  tqgetr("ac", iSi, 0, &result, &noerr);

  printf("Activity of pure Si is %g\n", result); 


  return 0;
 
}


