

/* Program cac17 */
/* Equilibrium calculations using streams */

#include "cacint.h"

int main()
{
  LI noerr, iSiO2, iO2, iCO2, i, nphase;
  DB TP_sol[2], TP_gas[2], vals[2], amount, value;
  char pname[TQSTRLEN], sname[TQSTRLEN];

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);

  /* Two input streams will subsequently be defined: The first one,
     labelled "STREAM_QUARTZ", will be solid SiO2(quartz) at 500 K,
     the second one, labelled "STREAM_GAS", will consist of a gas
     phase at room temperature consisting of O2 and CO2. By using
     streams instead of global conditions we will be able to get
     information about the extensive property balance of this
     process. */

  /* Define the first stream ("STREAM_QUARTZ") */
    
  /* Use the array called TP_sol to pass the information on temperature and
     pressure (500 K and 1 bar) of the stream to tqsttp */
  TP_sol[0] = 500.0;
  TP_sol[1] = 1.0;
  tqsttp("STREAM_QUARTZ", TP_sol, &noerr);

  /* Define the amounts of constituents of this stream 
     (1 mol quartz)*/
  tqinp("SiO2(quartz)", &iSiO2, &noerr);
  tqstca("STREAM_QUARTZ", iSiO2, 0, 1.0, &noerr);

  /* Define second stream ("STREAM_GAS") */
  TP_gas[0] = 298.15;
  TP_gas[1] = 1.0;
  tqsttp("STREAM_GAS", TP_gas, &noerr);
    
  /* Define the amounts of constituents of this stream (0.5 mol O2/GAS/ and
     0.5 mol CO2/GAS/) */
  tqinpc("O2", 1, &iO2, &noerr);
  tqstca("STREAM_GAS", 1, iO2, 0.5, &noerr);

  tqinpc("CO2", 1, &iCO2, &noerr);
  tqstca("STREAM_GAS", 1, iCO2, 0.5, &noerr);

  /* Set the temperature at which the two streams should react */
  tqstec("T", 0, 2500.0, &noerr);

  /* Define a third stream for the sole purpose of demonstrating how
     to remove it subsequently */
  tqsttp("DUMMY", TP_gas, &noerr);
  tqstca("DUMMY", 1, 1, 1.0, &noerr);

  /* Display what we have set so far */
  tqshow(&noerr);


  /* Remove the "DUMMY" stream */
  tqstrm("DUMMY", &noerr);
    
  /* Call tqshow again to demonstrate that the last stream has been removed */
  tqshow(&noerr) ;


  /* Calculate the equilibrium */
  tqcel(" ", 0, 0, vals, &noerr);

  /* Get the enthalpies of the two streams (note that as of version
     3.2.0 of ChemApp, TQSTXP can also be called _before_ the
     equilibrium calculation) */
  strcpy(sname,"STREAM_QUARTZ");
  tqstxp(sname, "H", &value, &noerr);
  printf("Enthalpy of stream %-24s: %g J\n", sname, value);
  strcpy(sname,"STREAM_GAS");
  tqstxp(sname, "H", &value, &noerr);
  printf("Enthalpy of stream %-24s: %g J\n", sname, value);


  /* Get the list of stable phases */
  printf("List of stable phases:\n");
  tqnop(&nphase, &noerr);
  for (i = 1; i <= nphase; i++) {
    tqgetr("A", i, 0, &amount, &noerr);
    if (amount > 0.0) {
      tqgnp(i, pname, &noerr);
      printf("Phase: %-24s amount: %g mol\n", pname, amount);
    }
  }


  /* Get the changes in the extensive property values in the current units */
  printf("Changes in extensive properties:\n");
  tqgetr("CP", 0, 0, &value, &noerr);
  printf("Delta CP: %g J/K\n", value);
  tqgetr("H", 0, 0, &value, &noerr);
  printf("Delta H : %g J\n", value);
  tqgetr("S", 0, 0, &value, &noerr);
  printf("Delta S : %g J/K\n", value);
  tqgetr("G", 0, 0, &value, &noerr);
  printf("Delta G : %g J\n", value);
  tqgetr("V", 0, 0, &value, &noerr);
  printf("Delta V : %g dm^3\n", value);


  /* To retrieve the *total* volume, use the option "VT" */
  tqgetr("VT", 0, 0, &value, &noerr);
  printf("Total volume: %g dm^3\n", value);


  return 0;
 
}


