

/* Program cac19 */
/* Retrieving results from equilibrium calculations */

#include "cacint.h"

int main()
{
  LI noerr, iSiO2, iCO2, indexp, nsc;
  DB TP[2], vals[2], value, values[3];
  char scname[TQSTRLEN];

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);


  /* Define a sample input stream ("INPUTS") */
    
  /*  Use the array called TP to pass the information on temperature and
      pressure (298.15 K and 1 bar) of the stream to tqsttp */
  TP[0] = 298.15 ;
  TP[1] = 1.0;
  tqsttp("INPUTS", TP, &noerr);

  /* Define the amounts of constituents of this stream (1 mol quartz and
     0.5 mol CO2) */
  tqinp("SiO2(quartz)", &iSiO2, &noerr);
  tqstca("INPUTS", iSiO2, 0, 1.0, &noerr);
  tqinpc("CO2", 1, &iCO2, &noerr);
  tqstca("INPUTS", 1, iCO2, 0.5, &noerr);

  /* Set the temperature at which the two streams should react */
  tqstec("T", 0, 2500.0, &noerr);

  /* Calculate the equilibrium */
  tqcel(" ", 0, 0, vals, &noerr);


  /* A number of the results retrieved below can also be found in the
     ChemSage result table output above by tqcel. If you do not want a
     ChemSage result table, for instance because your application
     program has a graphical user interface and cannot make use of the
     table, call tqce instead to calculate the equilibrium. */

  /* Get the pressure, which is 1 bar, since this is the default. */
  tqgetr("P", 0, 0, &value, &noerr);
  printf("Equilibrium pressure: %g bar\n", value);

    
  /* Get the calculated equilibrium volume */
  tqgetr("VT", 0, 0, &value, &noerr);
  printf("Equilibrium volume (total): %g dm^3\n", value);


  /* Get the calculated _change_ in volume. Note that options "V" and
     "VT" _only_ give different values if streams are used _and_ the
     whole system is selected, in which case "V" retrieves the
     _change_ in volume between the reaction products and the stream
     constituents. In all other cases, both "VT" and "V" give the
     _total_ volume. */
  tqgetr("V", 0, 0, &value, &noerr);
  printf("Change in volume: %g dm^3\n", value);


  /* Get the temperature, which is 2500 K, since we set this particular
     condition for the equilibrium calculation */
  tqgetr("T", 0, 0, &value, &noerr);
  printf("Equilibrium temperature: %g K\n", value);


  /* Get the equilibrium amount of liquid SiO2 */
  tqinp("SiO2(liquid)", &indexp, &noerr);
  tqgetr("a", indexp, 0, &value, &noerr);
  printf("Equilibrium amount of SiO2(liquid): %g mol\n", value); 


  
  /* Get the incoming amount of CO2/GAS/ that was input for the last
     equilibrium calculation */
  tqgetr("ia", 1, iCO2, &value, &noerr);
  printf("Incoming amount of CO2/GAS/: %g mol\n", value);


  /* Get the chemical potential of SiO2(quartz) */
  tqgetr("mu", 1, iSiO2, &value, &noerr);
  printf("Chemical potential of SiO2(quartz): %g J/mol\n", value); 


  /* Get the activity of SiO2(quartz) */
  tqgetr("ac", 1, iSiO2, &value, &noerr);
  printf("Activity of SiO2(quartz): %g\n", value);


  /* Get the activity of the first system component (carbon). Note that the
     activities of system components are given in absolute values, whereas
     relative values are obtained for phases and constituents. */
  tqgetr("ac", 0, 1, &value, &noerr);
  printf("Activity of system component C: %g\n", value); 


  /* Get the changes in the extensive property values between the
     reaction products and the reactants given as the stream
     constituents in the current units for the whole system (note that
     these are relative values) */
  printf("Changes in extensive properties:\n");
  tqgetr("CP", 0, 0, &value, &noerr);
  printf("Delta CP: %g J/K\n", value);
  tqgetr("H", 0, 0, &value, &noerr);
  printf("Delta H: %g J\n", value);
  tqgetr("S", 0, 0, &value, &noerr);
  printf("Delta S: %g J/K\n", value);
  tqgetr("G", 0, 0, &value, &noerr);
  printf("Delta G: %g J\n", value);
  tqgetr("V", 0, 0, &value, &noerr);
  printf("Delta V: %g dm^3\n", value);


  /* Get the Gibbs energy value for CO2/GAS/ */
  tqgetr("G", 1, iCO2, &value, &noerr);
  printf("G(CO2/GAS/): %g J\n", value);


  /* Get the Gibbs energy value for CO2/GAS/ per amount unit */
  tqgetr("GM", 1, iCO2, &value, &noerr);
  printf("G_M(CO2/GAS/): %g J/mol\n", value);


  /* Get the mole fraction of system component C in the system */
  tqgetr("X", 0, 1, &value, &noerr);
  printf("Mole fraction of C in the system: %g\n", value);


  /* Get the mole fraction of system component C in the gas phase */
  tqgetr("XP", 1, 1, &value, &noerr);
  printf("Mole fraction of C in the gas phase: %g\n", value);


  /* Now get the total amount of all system components in the gas
     phase, having tqgetr return all three values in one array */
  tqgetr("ap", 1, -1, values, &noerr); 
  printf("Amounts of all system components in the GAS phase:\n");

  for (nsc = 0; nsc < 3; nsc++) {

    tqgnsc(nsc+1, scname, &noerr);
    printf("Amount of %s in the GAS phase: %g mol\n",
	   scname, values[nsc]);

  }


  return 0;
 
}


