

/* Program cac20 */
/* Calculate thermodynamic quantities */

#include "cacint.h"

int main()
{
  LI noerr, numcon, iSiO2;
  DB value, R, T;

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);

  /* Defining the gas constant in J/mol.K */
  R = 8.31451;

  /* Get thermochemical function values for SiO2(quartz) at 500 K */
  T = 500.0;
  tqsetc("T", 0, 0, T, &numcon, &noerr);
  tqinp("SiO2(quartz)", &iSiO2, &noerr);

  /* Note that when retrieving CP, H, S, or G, the values are returned
     dimensionless, which means they might have to be multiplied with
     R*T. Since the default amount unit is mol, results are returned
     for 1 mol. Note also that care has to be taken if a temperature
     unit different from Kelvin has been used. */
  tqgdpc("CP", iSiO2, 0, &value, &noerr);
  printf("CP of SiO2(quartz) at 500 K: %g J/mol.K\n", value*R*T);

  tqgdpc("H", iSiO2, 0, &value, &noerr);
  printf("H of SiO2(quartz) at 500 K: %g J/mol\n", value*R*T);


  /* The option "T" serves a special purpose: If used, it returns the
     end temperature of the current interval of the CP or G function
     for the selected phase or phase constituent. Looking at the
     data-file cosi.dat (see Appendix ref(cosidat)), one sees from the
     entry for SiO2(quartz) that for a temperature of 500 K, the
     temperature range of the G function in question extends to 848
     K. */
  tqgdpc("T", iSiO2, 0, &value, &noerr);
  printf("For SiO2(quartz) and T = 500 K, the present\n"
	 "temperature interval ends at: %g K\n", value);


  /* Setting a different temperature for which a different G function
     interval applies */
  T = 1000.0;
  tqsetc("T", 0, 0, T, &numcon, &noerr);
  tqgdpc("T", iSiO2, 0, &value, &noerr);
  printf("For SiO2(quartz) and T = 1000 K, the present\n"
	 "temperature interval ends at: %g K\n", value);


  /* The option "T" of tqgdpc is especially useful when the
     thermodynamic values are calculated as a function of temperature,
     and the ranges of the CP or G functions need to be known
     explicitly in order to locate transition temperatures between
     different modifications and determine the associated changes in
     the thermodynamic values. */

  return 0;
 
}


