

/* Program cac22 */
/* Change the limits of target variables */

/* Note that this program contains target calculations,
   which cannot be performed with the 'light' version of ChemApp */ 

#include "cacint.h"

int main()
{
  LI noerr, islite, numcon, iPb, iSn, iLIQ, iFCC;
  DB temp, act, vals[2];

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Check if we are working with the 'light' version.
     If we do, omit the following target calculation(s). */
  tqlite(&islite, &noerr);
  if (islite) 
    {
      printf("*** Target calculations have been omitted here,\n"
             "*** since they are not possible with the\n"
             "*** 'light' version of ChemApp.\n\n");
    }
  else 
    {


  /* Open data-file for reading */
  tqopna("pbsn.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);

  /* For this example, let us assume one intends to find the solidus
     point for an Pb-Sn alloy consisting of 20 mol-% Sn and 80 mol-%
     Pb by performing a precipitation target calculation from the FCC
     phase. Figure ref(pbsn2_pd) shows the Pb-Sn phase diagram as
     calculated by ChemSage. The line marked "1" indicates the
     concentration that will be used for the precipitation target
     calculations below, using the temperature as the target
     variable. */

  /* Get index numbers of system components */
  tqinsc("Pb", &iPb, &noerr);
  tqinsc("Sn", &iSn, &noerr);

  /* Get phase index numbers of the FCC and liquid phases */
  tqinp("FCC", &iFCC, &noerr);
  tqinp("LIQ", &iLIQ, &noerr);

  /* Input 0.2 mol Sn and 0.8 mol Pb */
  tqsetc("ia", 0, iSn, 0.2, &numcon, &noerr);
  tqsetc("ia", 0, iPb, 0.8, &numcon, &noerr);

  /* Define the precipitation target */
  tqsetc("a", iFCC, 0, -1.0, &numcon, &noerr);

  /* Define the target variable (temperature) and perform the
     equilibrium calculation. The estimate for the target variable,
     passed C via vals[0], is set to 1000 K. */
  vals[0] = 1000.0;
  tqce("T", 0, 0, vals, &noerr); 

  /* Retrieve the calculated temperature */
  tqgetr("T", 0, 0, &temp, &noerr);
  printf("Calculated solidus temperature: %g K\n", temp);


  /* Looking at the Pb-Sn phase diagram, this temperature is certainly
     wrong. This can be demonstrated by retrieving the activity of the
     liquid phase: */
  tqgetr("ac", iLIQ, 0, &act, &noerr);
  printf("Activity of liquid phase at %g K: %g\n", temp, act);


  /* Since the activity is less than unity, the liquid phase is not
     stable, thus this cannot be the solidus point. The reason is that
     ChemApp found the phase boundary between FCC and the two-phase
     region FCC+BCT, instead of that between FCC and FCC+Liquid. Note
     also that although the initial estimate for the target variable
     has been set to 1000 K, which is certainly closer to the solidus
     point than to the FCC/FCC+BCT phase boundary, this does not
     guarantee that of two possible solutions, the one closer to the
     initial estimate is found. */

  /* To keep ChemApp from finding the unwanted phase boundary at lower
     temperatures, tell it to limit its search to temperatures above
     450 K: */
  tqclim("TLOW", 450.0, &noerr);
  
  /*  Recalculate the equilibrium */
  tqce("T", 0, 0, vals, &noerr); 

  /* Retrieve the calculated temperature */
  tqgetr("T", 0, 0, &temp, &noerr);
  printf("Calculated solidus temperature: %g K\n", temp);


  /* Check the activity of the liquid phase again to verify that it is
     indeed stable: */ 
  tqgetr("ac", iLIQ, 0, &act, &noerr);
  printf("Activity of liquid phase at %g K: %g\n", temp, act);
 

  /* Since the activity is unity, we know that the temperature we
     found is correct. */
    
  }
  return 0;
 
}

