


/* Program cac24 */
/* One-dimensional phase mapping calculation */

/* Note that this program contains target calculations,
   which cannot be performed with the 'light' version of ChemApp */ 

#include "cacint.h"

int main()
{
  LI noerr, numcon, islite, iSiO2, resno=0, icont;
  DB vals[2], result;

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading. */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);

  /* Check if we are working with the 'light' version.
     If we do, omit the following target calculation(s). */
  tqlite(&islite, &noerr);
  if (islite) 
    {
      printf("*** Target calculations have been omitted here,\n"
             "*** since they are not possible with the\n"
             "*** 'light' version of ChemApp.\n\n");
    }
  else 
    {


  /* Determine the index number for the phase SiO2(quartz) */
  tqinp("SiO2(quartz)", &iSiO2, &noerr);

  /* Enter one mol of SiO2 */
  tqsetc("IA", iSiO2, 0, 1.0, &numcon, &noerr);


  /* The temperature search interval is supposed to range from 300 to
     3000 K: */
  vals[0] = 300.0;
  vals[1] = 3000.0;

  /* First call to tqmap, note the "f" ("first") in the option
     parameter */
  tqmap("tf", 0, 0, vals, &icont, &noerr);

  /* The variable resno keeps track of the number of times we call
     tqmap: */
  resno++;

  /* Retrieve and print the temperature, which we know is vals[0]: */
  tqgetr("t", 0, 0, &result, &noerr);
  printf("*** Lower interval boundary: %g K\n", result);

  /* For as long as icont is positive, we need to make further calls
     of tqmap */
  while (icont) {

    /* tqmap is called again. Note the "n" ("next") in the option
       parameter. If we are at the first phase boundary (resno is
       still 2), we call tqmapl for a change to produce a ChemSage
       output table... */
    if (resno == 2) {
      printf("*** ChemSage result table "
	     "for the first phase boundary found:\n");
      tqmapl("tn", 0, 0, vals, &icont, &noerr);
      printf("\n");

      /* ...otherwise we just call tqmap: */
    } else {
      tqmap("tn", 0, 0, vals, &icont, &noerr);
    }
    resno++;

    /* Get the temperature... */
    tqgetr("t", 0, 0, &result, &noerr);

    /* ...and print the entry for the result table. If we have called
       tqmap twice already, we know that we have found a phase
       boundary. If not, we have retrieved the temperature value of
       the upper interval boundary (vals[1]): */

    if (resno > 2) {
      printf("*** Phase boundary found at %g K\n", result);
    } else {
      printf("*** Upper interval boundary: %g K\n", result);
    }


  }



  /* With the above example the temperatures of all phase boundaries in a
     system which contains 1 mol of SiO2 have been calculated. Thus the
     phase boundaries determined reflect the stability ranges of the
     various modifications of SiO2. Also note that the first two
     temperatures determined are _no_ phase boundaries, but the lower and
     upper limit of the search interval (vals[0] and vals[1]).

     For the first phase boundary found, tqmapl is called instead of
     tqmap, thus producing a ChemSage output table. From this ChemSage
     output table it can be seen that at the temperature calculated
     SiO2 transforms from the quartz into the tridymite modification.

     Refer to the worked example 5 (One-dimensional phase mapping) for
     a further example. This also demonstrates how to obtain
     information on which phases are stable at each of the phase
     boundaries determined. */

  }

  return 0;
 
}


