

/* Program cac25 */
/* Determining whether phase constituents are permitted as incoming
   species */

#include "cacint.h"

int main()
{
  LI noerr, nphase, npcon, i, j, inpcis, nnperm;
  char pname[TQSTRLEN], pcname[TQSTRLEN];

  /* Initialise the number of phase constituents which cannot be used as
     incoming species to 0 */
  nnperm = 0;
	
  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading */
  tqopna("cosi.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);
 
  /* Check whether the data-file contains any phase constituents that
     cannot be used as incoming species. */

  /* Get the number of phases */
  tqnop(&nphase, &noerr);

  /* Print a table header */
  printf("%-25s%-25s%-19s\n", "PHASE", "PHASE CONSTITUENT",
	 "O.K./NOT PERMITTED");

  /* Loop over all phases */
  for (i=1; i<=nphase; i++) {
    tqgnp(i, pname, &noerr);
    tqnopc(i, &npcon, &noerr);
    
    /* We only need to check mixture phases */
    if (npcon > 1) {
      /* Loop over all constituents */
      for (j=1; j<=npcon; j++) {
	tqgnpc(i, j, pcname, &noerr);
	/* Check whether the current constituent is permitted to be
	   used as incoming species */
	tqpcis(i, j, &inpcis, &noerr);
	
	/* Print a table entry, and if it is not permitted, increase the total
	   number of such phase constituents by 1 */
	if (inpcis == 0) {
	  nnperm++;
	  printf("%-25s%-25s%-19s\n", pname, pcname, "not permitted");
	} else {
	  printf("%-25s%-25s%-19s\n", pname, pcname, "o.k.");
	}
      }
    }
  }
  
  /* If there were phase constituents which cannot be used as incoming
     species, print a note. */
  if (nnperm > 0) {
    printf("Note: %li phase constituent(s) is/are not permitted\n"
	   "      as incoming species.\n", nnperm);
  }
  

  
  return 0;
  
}


