

/* Program cac26 */
/* Handling sublattice constituents */

#include "cacint.h"

int main()
{

      
  LI noerr, nphase, nsl, nslcon, indexp, indexl, indexc, i;
  char pname[TQSTRLEN], cname[TQSTRLEN];

  /* Initialise ChemApp */
  tqini(&noerr);

  /* Open the thermochemical data-file subl-ex.dat for reading. It
     contains an extract of the system Co-Cr-Fe: the phase SIGMA:30,
     which is modelled according to the sublattice formalism, and the
     BCC phase, described by a Redlich-Kister polynomial. Both phases
     are each included twice, to account for miscibility gaps. */
  tqopna("subl-ex.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);

  /* Close data-file */
  tqclos(10, &noerr);
      
  /* The first of the two identical copies of the SIGMA:30 phase,
     which ChemApp calls SIGMA:30#1, will be investigated with respect
     to the number of sublattices, the number of sublattice
     constituents on each sublattice, and the names of the sublattice
     constituents. */

  /* Get the index number for the phase SIGMA:30#1 */
  strcpy(pname, "SIGMA:30#1");
  tqinp(pname, &indexp, &noerr);

  /* Get the number of sublattices */
  tqnosl(indexp, &nsl, &noerr);
  printf("%s has %li sublattices\n",pname, nsl);

            
  /* Loop over all sublattices */
  for (indexl = 1; indexl<=nsl; indexl++) {

    /* Get the number of sublattice constituents */
    tqnolc(indexp, indexl, &nslcon, &noerr);
    printf("Sublattice %li has %li constituents with the following names:\n",
	   indexl, nslcon);

    /* Get the name of each sublattice constituent */
    for (indexc = 1; indexc<=nslcon; indexc++) {
      tqgnlc(indexp, indexl ,indexc, cname, &noerr);

      /* The reverse (getting the index number for the name of the
	 sublattice constituent just retrieved), is rather superfluous
	 here, and only used to demonstrate the call to tqinlc: */
      tqinlc(cname, indexp, indexl, &i, &noerr);
      printf("   %li: %s\n", i, cname);
    }
  }

  
  return 0;
  
}


