

/* Program cac27 */
/* Retrieving results for sublattice constituents */

#include "cacint.h"

int main()
{

  LI noerr, islite, numcon, i, indexp, indexl, indexc, nphase, nsl, nslcon;
  DB val, vals[2], result;
  char pname[TQSTRLEN], mname[TQSTRLEN], cname[TQSTRLEN];

  /* Initialise ChemApp */
  tqini(&noerr);

  /* Open the thermochemical data-file subl-ex.dat for reading. It
     contains an extract of the system Co-Cr-Fe: the phase SIGMA:30,
     which is modelled according to the sublattice formalism, and the
     BCC phase, described by a Redlich-Kister polynomial. Both phases
     are each included twice, to account for miscibility gaps. */
  tqopna("subl-ex.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);

  /* Close data-file */
  tqclos(10, &noerr);


  /* Set the temperature to 1000 K */
  tqsetc("T", 0, 0, 1000.0, &numcon, &noerr);

  /* Set the incoming amounts to 0.25 mol Co, 0.25 mol Cr, 
     and 0.5 mol Fe */
  tqinsc("Co", &indexc, &noerr);
  tqsetc("ia", 0, indexc, 0.25, &numcon, &noerr);

  tqinsc("Cr", &indexc, &noerr);
  tqsetc("ia", 0, indexc, 0.25, &numcon, &noerr);

  tqinsc("Fe", &indexc, &noerr);
  tqsetc("ia", 0, indexc, 0.50, &numcon, &noerr);

  /* Calculate the equilibrium */
  tqcel(" ", 0, 0, vals, &noerr);


  /* As can be seen in the output above, the standard ChemSage result
     table obtained with TQCEL contains a section with details on the
     stable sublattice phases. The following code shows how to
     retrieve similar information with ChemApp. */

  /* Loop over all phases, and if a phase is stable and a sublattice
     phase, print information about the sublattices */
  tqnop(&nphase, &noerr);
  for (indexp = 1; indexp<=nphase; indexp++) {

    /* Check if the phase is stable, otherwise we don't need to
       consider it */
    tqgetr("a", indexp, 0, &val, &noerr);
    if (val > 0) {

      /* Get the name and model of the phase */
      tqgnp(indexp, pname, &noerr);
      tqmodl(indexp, mname, &noerr);

      /* If the model name of the phase starts with 'SUB' we have a
	 sublattice phase */
      if (strncmp(mname, "SUB", 3) == 0) {

	/* Print a header similar to the one in the ChemSage output
	   table */
	printf("Mole fraction of the sublattice constituents in %s\n",
	       pname);
    
	/* Get the number of sublattices */
	tqnosl(indexp, &nsl, &noerr);
            
	/* Loop over all sublattices */
	for (indexl=1; indexl<=nsl; indexl++) {

	  printf("Sublattice %li\n", indexl);
	  /* Get the number of sublattice constituents */
	  tqnolc(indexp, indexl, &nslcon, &noerr);

	  for (indexc=1; indexc<=nslcon; indexc++) {
	    /* Get the name of each sublattice constituent... */
	    tqgnlc(indexp, indexl, indexc, cname, &noerr);

	    /* ... and its mole fraction in the sublattice */
	    tqgtlc(indexp, indexl, indexc, &val, &noerr);
                  
	    printf("%-25s %f\n", cname, val);
	  }
	}
      }
    }
  }



  return 0;

}

