

/* Program cac31 */
/* Read a thermochemical data-file in TRANSPARENT format, then
   retrieves information on the transparent file header. */

/* NOTE: This example program requires a data-file in transparent format 
         to run, which is not part of a regular distribution of ChemApp
         or ChemApp "light". */

#include "cacint.h"

int main()
{
  LI noerr;
 
  /* Variables to store information from various transparent file
     header (TFH) fields */
  char tfhid[255], tfhusr[80],tfhnwp[40],tfhnrp[40],tfhrem[80];
  LI tfhver, tfhvnw[3], tfhvnr[3], tfhdtc[6], tfhdte[6];

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Note that tqopnt is used to open the thermochemical data-file,
     instead of a standard C library routine like fopen. The reason
     is that it is necessary to associate the data-file with a
     particular FORTRAN unit number (10 by default, see tqgio) so
     that the FORTRAN subroutine tqrcst can read from the correct
     file */
  tqopnt("cosiex.cst",10,&noerr);

  /* Read data-file */
  tqrcst(&noerr);
  
  /* Close data-file */
  /* Again, the routine for closing the data-file is not a standard
     C library routine like fclose, but a special one to make sure the
     file that was previously opened under the unit number specified
     is closed. */
  tqclos(10,&noerr);
   
  /* Once the transparent data-file has been read, information on its 
     header can be retrieved.*/
  tqgtrh(&tfhver, tfhnwp, tfhvnw, tfhnrp, tfhvnr, tfhdtc, tfhdte, 
	 tfhid, tfhusr, tfhrem, &noerr);

  printf("Version number of the transparent file header format: %li\n", 
	 tfhver);
  printf("Name of the program which wrote the data-file: %s\n", tfhnwp);
  printf("Version number of the writing program: %li.%li.%li\n", 
	 tfhvnw[0],tfhvnw[1],tfhvnw[2]);
  printf("Programs which are permitted to read the data-file: %s\n", tfhnrp);
  printf("Min. version number of the reading program: %li.%li.%li\n", 
	 tfhvnr[0],tfhvnr[1],tfhvnr[2]);
  printf("File was created on %li/%02li/%02li %02li:%02li:%02li \n", 
	 tfhdtc[0],tfhdtc[1],tfhdtc[2],tfhdtc[3],tfhdtc[4],tfhdtc[5]);
  printf("File will expire on %li/%02li/%02li %02li:%02li:%02li \n", 
	 tfhdte[0],tfhdte[1],tfhdte[2],tfhdte[3],tfhdte[4],tfhdte[5]);
  printf("Licensee's user ID(s): %s\n", tfhid);
  printf("Licensee's name: %s\n", tfhusr);
  printf("Remarks : %s\n", tfhrem);

  
  return 0;
 
}


