

C Print data-file contents

      PROGRAM CAF100
      IMPLICIT NONE
      
      INTEGER NOERR, I, J, NSCOM, NPHASE, NPCON, INPCIS, NNPERM
      CHARACTER FNAME*12, SCNAME*24, PNAME*24, MODEL*6, PCNAME*24

C Initialise the number of phase constituents which cannot be used as
C incoming species to 0
      NNPERM = 0

C Initialise ChemApp 
      CALL TQINI(NOERR)

      FNAME = 'cosi.dat'
C For this program to be flexible, add code that asks for the name of
C the data-file instead

C Open the thermochemical data-file cosi.dat (system C-O-Si) C for
C reading
      CALL TQOPNA(FNAME, 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)
      
      WRITE(*,FMT='(1X,A)') 'Contents of data-file ' // FNAME

C ***************************************************************
C Information on system components
C ***************************************************************

C Get the number of system components
      CALL TQNOSC(NSCOM, NOERR)

      WRITE(*,FMT='(1X,A,I3)') 'Number of system components: ',
     *     NSCOM 

C Print the names of all system components
      WRITE(*,FMT='(1X,A)') 'List of system components:'

      DO I=1, NSCOM
         CALL TQGNSC(I, SCNAME, NOERR)
         WRITE(*,FMT='(1X,I4,A)') I, ' : ' // SCNAME
      ENDDO
      WRITE(*,*)


C ***************************************************************
C Information on phases
C ***************************************************************

C Get the number of phases

      CALL TQNOP(NPHASE, NOERR)

      WRITE(*,FMT='(1X,A,I3)') 'Number of phases: ',
     *     NPHASE 
      
C Print the names of all phases, the associated model, and the number of
C phase constituents, if it is a mixture phase.
      WRITE(*,FMT='(1X,A)') 'List of phases:'

      DO I=1, NPHASE

C Get the phase name          
         CALL TQGNP(I, PNAME, NOERR)

C Get the model name
         CALL TQMODL(I, MODEL, NOERR)

C Get its number of phase constituents
         CALL TQNOPC(I, NPCON, NOERR)

C If this phase has more than 1 phase constituent, it is a mixture
C phase...
         IF (NPCON .GT. 1) THEN
            WRITE(*,FMT='(1X,I4,A,I4,A)') I, ' : ' // PNAME // ' ' // 
     *           MODEL, NPCON, ' constituents'

C ... otherwise it is a stoichiometric compound
         ELSE
            WRITE(*,FMT='(1X,I4,A)') I, ' : ' // PNAME // ' ' // MODEL  
         ENDIF
      ENDDO
      WRITE(*,*)



C ***************************************************************
C Information on phase constituents of mixture phases
C ***************************************************************

C Print information on phase constituents
      WRITE(*,FMT='(1X,A)') 'List of phase constituents of ' // 
     *     'mixture phases:'

      DO I=1, NPHASE
C Get its number of phase constituents
         CALL TQNOPC(I, NPCON, NOERR)

C If this phase has more than 1 phase constituent, it is a mixture
C phase, and we want to print the list of phase constituents
         IF (NPCON .GT. 1) THEN
            CALL TQGNP(I, PNAME, NOERR)
            WRITE(*, FMT='(1X,A)') 'List of phase constituents of ' //
     *           'phase ' // PNAME

C Get and print the name of each phase constituent of the current phase
            DO J=1, NPCON
               CALL TQGNPC(I, J, PCNAME, NOERR)
C Check whether the phase constituent may be used as incoming
C species. If not, increase the total number of such phase constituents
C by 1 and print a mark.
               CALL TQPCIS(I, J, INPCIS, NOERR)
               IF (INPCIS .EQ. 0) THEN
                  NNPERM = NNPERM + 1
                  WRITE(*,FMT='(1X,I4,A)') J, ' : ' // PCNAME // ' X'
               ELSE
                  WRITE(*,FMT='(1X,I4,A)') J, ' : ' // PCNAME
               ENDIF
            ENDDO
            WRITE(*,*)

         ENDIF
      ENDDO

C If there have been phase constituents which cannot be used as incoming
C species, print a note.
      IF (NNPERM .GT. 0) THEN
         WRITE(*,FMT='(1X,A, I3,A)') 'Note: ', NNPERM, ' phase ' //
     *        'constituent(s) marked with "X" is/are not permitted '
         WRITE(*,FMT='(1X,A)') '        as incoming species.'
      ENDIF



      END

