
C Equilibrium composition of mixture phases

      PROGRAM CAF101
      
      IMPLICIT NONE
      
      CHARACTER NAME*24

      DOUBLE PRECISION STOI(30), VALS, VALUE, AGAS, ASYS, VALUES(50),
     *     FRACSI, SITOT, WMASS, XSN, HINT1, HINT2, HPB, HSN

      INTEGER NOERR, NUMCON, ICO, ISIO2, NPCON, I, NSCOM, ISI,
     *     IPB, ISN, ILIQ

C Initialise ChemApp 
      CALL TQINI(NOERR)

C Open the thermochemical data-file cosi.dat (system C-O-Si)
C for reading
      CALL TQOPNA('cosi.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C For the present example, the input composition consists of 2 mol of
C CO/GAS/ and 3 mol of quartz
      CALL TQINPC('CO ', 1, ICO, NOERR)
      CALL TQSETC('IA ', 1, ICO ,2.D0, NUMCON, NOERR)

      CALL TQINP('SiO2(quartz)', ISIO2, NOERR)
      CALL TQSETC('IA ', ISIO2, 1, 3.D0, NUMCON, NOERR)

C The temperature at which the equilibrium is to be calculated is set to
C 1500 K
      CALL TQSETC('T ', 0, 0, 1500.D0, NUMCON, NOERR)

C Calculate the equilibrium and print a ChemSage result table
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


C First, get the equilibrium amount of the gas phase, which is the only
C mixture phase in the system. Note that since we did not change any
C units, the default amount unit (mol) is used.
      CALL TQGETR('A ', 1, 0, AGAS, NOERR)

      WRITE(*,FMT='(A,G12.5)') 'Equilibrium amount of gas phase ' //
     *     'in mol: ', AGAS


C To determine the mole fraction of the gas phase, relative to the whole
C system, we first need to get the amount contained in the system
      CALL TQGETR('A ', 0, 0, ASYS, NOERR)

      WRITE(*,FMT='(A,G12.5)') 'Mole fraction of the gas phase: ',
     *     AGAS/ASYS


C The equilibrium amounts of all phase constituents of the gas phase can
C be obtained by a call to TQGETR which passes an array together with
C the proper INDEX/INDEXP combination
      CALL TQGETR('A ', 1, -1, VALUES, NOERR)

C Print the equilibrium amount of each phase constituent
      CALL TQNOPC(1, NPCON, NOERR)
      WRITE(*,*) 'Equilibrium amounts of gas phase constituents'
      DO I=1, NPCON
         CALL TQGNPC(1, I, NAME, NOERR)
         WRITE(*,FMT='(I4,A,G12.5,A)') I, ' : ' // NAME, 
     *        VALUES(I), ' mol'
      ENDDO


C The mole fractions of the phase constituents can be calculated using
C the values just retrieved

C Print the mole fractions of each phase constituent
      WRITE(*,*) 'Mole fractions of gas phase constituents'
      DO I=1, NPCON
         CALL TQGNPC(1, I, NAME, NOERR)
         WRITE(*,FMT='(I4,A,G12.5)') I, ' : ' // NAME, 
     *        VALUES(I)/AGAS
      ENDDO


C Information relating the system components (in the present case C, O,
C and Si) to the phases can also be obtained.

C Get the mole fraction of the system components in the gas phase
      CALL TQNOSC(NSCOM, NOERR)
      WRITE(*,*) 'Mole fractions of system components in the gas phase'
      DO I=1, NSCOM
         CALL TQGNSC(I, NAME, NOERR)
         CALL TQGETR('XP ', 1, I, VALUE, NOERR)
         WRITE(*,FMT='(I4,A,G12.5)') I, ' : ' // NAME, 
     *        VALUE
      ENDDO

         
C To get the fraction of a system component in the gas phase, we have to
C make use of the information contained in the stoichiometry matrix.
      
      CALL TQINSC('Si ', ISI, NOERR)
      FRACSI = 0.D0

C Get the equilibrium amount of each gas phase constituent, then use the
C stoichiometry matrix returned by TQSTPC to determine how much Si is
C contained in the constituent, and sum it all up
      DO I = 1, NPCON
         CALL TQSTPC(1, I, STOI, WMASS, NOERR)
         CALL TQGETR('A ', 1, I, VALUE, NOERR)
         FRACSI = FRACSI + STOI(ISI)*VALUE
      ENDDO

C Get the total amount of Si in the system
      CALL TQGETR('a ',0,ISI,SITOT,NOERR)

      WRITE(*,FMT='(A,G12.5)') 'Mole fraction of system component ' //
     *     'Si in the gas phase: ', FRACSI/SITOT



C To demonstrate how ChemApp can be used to check the relationship
C between partial and integral values of extensive properties, a
C different data-file for a system that contains non-ideal mixture
C phases is used.

C Open a new thermochemical data-file (pbsn.dat, system Pb-Sn)
C for reading
      CALL TQOPNA('pbsn.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Find out the system component index numbers for Pb and Sn, as well as
C the phase index number for the liquid phase:
      CALL TQINSC('Pb ', IPB, NOERR)
      CALL TQINSC('Sn ', ISN, NOERR)
      CALL TQINP('LIQUID ', ILIQ, NOERR)

C Using the index numbers just determined, we can enter our incoming
C amounts, in this case an alloy consisting of 0.7 mol Sn and 0.3 mol Pb:
      XSN = 0.7D0
      CALL TQSETC('IA ', 0, ISN, XSN, NUMCON, NOERR)
      CALL TQSETC('IA ', 0, IPB,1.D0 - XSN, NUMCON, NOERR)
      
C Set the temperature to 600 K. Under these conditions, the liquid phase
C is the only stable phase in this system (see Figure ref(pbsn2_pd)).
      CALL TQSETC('T ', 0, 0, 600.D0, NUMCON, NOERR)

C Calculate the equilibrium
      CALL TQCE(' ', 0, 0, VALS, NOERR) 

C Get the integral enthalpy of the liquid phase
      CALL TQGETR('H ', ILIQ, 0, HINT1, NOERR) 

C Get the partial molar enthalpies of Sn/LIQUID/ and Pb/LIQUID/
      CALL TQGETR('HM ', ILIQ, ISN, HSN, NOERR) 
      CALL TQGETR('HM ', ILIQ, IPB, HPB, NOERR) 

C Calculate the integral enthalpy of the liquid phase from the two
C partial values
      HINT2 = XSN * HSN + (1.D0 - XSN) * HPB

C Print both values
      WRITE(*,FMT='(A,F12.3,A)') 'HINT1 = ', HINT1,' J'
      WRITE(*,FMT='(A,F12.3,A)') 'HINT2 = ', HINT2,' J'

C As expected, the two values are the same

      END
