
C Changing the status of phases

      SUBROUTINE TABLE
      IMPLICIT NONE

      INTEGER I, NOERR, NPHASE
      DOUBLE PRECISION AMOUNT, ACT
      CHARACTER NAME*24

C Retrieve and display the activities and equilibrium 
C amounts of all phases

C Get number of phases
      CALL TQNOP(NPHASE, NOERR)

C Print table header
         write(*,FMT='(A10,17X,A14,2X,A14)') 'Phase name',
     *     'amount/mol',
     *     'activity'

      DO I=1, NPHASE
C Get the phase name
         CALL TQGNP(I, NAME, NOERR)

C Get its equilibrium amount
         CALL TQGETR('A ',  I, 0, AMOUNT, NOERR)

C Get its activity 
         CALL TQGETR('AC ', I, 0, ACT, NOERR)
         write(*,FMT='(A24,2X,F14.5,2X,F14.5)') NAME,AMOUNT,ACT
      ENDDO
      RETURN
      END


      PROGRAM CAF14
      IMPLICIT NONE
      
      INTEGER NOERR, NUMCON
      INTEGER ISIC, ISIO2, IC
      DOUBLE PRECISION VALS(2)
      CHARACTER STATUS*24

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading 
      CALL TQOPNA('cosi.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Get the phase index number of SiC
      CALL TQINP('SiC ', ISIC, NOERR)
      
C Get the status of this phase
      CALL TQGSP(ISIC, STATUS, NOERR)

      WRITE(*,FMT='(A)') 'The status of SiC is currently ' //
     *     STATUS


C Unless a phase is marked "dormant" or "eliminated" in the data-file
C itself, the status is by default always "entered". This means it is
C considered in the subsequent equilibrium calculations and is included
C in the mass balance.

C Set some conditions and perform a simple calculation to 
C demonstrate the effect of the different statuses

C Set temperature to 1900 K
      CALL TQSETC('T ', 0, 0, 1900.D0, NUMCON, NOERR)

C Set incoming amount to 1 mol SiO2(quartz) and 2 mol C
      CALL TQINP('SiO2(quartz)', ISIO2, NOERR)
      CALL TQSETC('IA ', ISIO2, 0, 1.D0, NUMCON, NOERR)

      CALL TQINP('C ', IC, NOERR)
      CALL TQSETC('IA ', IC, 0, 2.D0, NUMCON, NOERR)

C Calculate the equilibrium
      CALL TQCE(' ', 0, 0, VALS, NOERR)

C Print a table containing the phase names, their
C equilibrium amounts, and activities
      CALL TABLE


C From the above table it can be seen that SiC is stable
C under those conditions. Its activity is unity and its 
C equilibrium amount is approx. 0.65 mol

C Now change the status of SiC to "dormant" and rerun the 
C calculation

      CALL TQCSP(ISIC, 'DORMANT ', NOERR)
      CALL TQCE(' ', 0, 0, VALS, NOERR)
      CALL TABLE


C Setting a phase to "dormant" tells ChemApp to exclude it from the mass
C balance, but still calculate its activity.  This offers the
C possibility to consider a phase to be metastable, while still
C obtaining information about its activity. It can be seen from the
C above table that ChemApp sets the equilibrium amount of SiC to zero,
C and calculated its activity to be almost 30. Instead of SiC, ChemApp
C now calculates Si to be stable.

C If a phase is to be completely ignored during the equilibrium
C calculations, its status should be set to "eliminated", as in the
C following example.

      CALL TQCSP(ISIC, 'ELIMINATED ', NOERR)
      CALL TQCE(' ', 0, 0, VALS, NOERR)
      CALL TABLE

      
C Now ChemApp ignored SiC during the equilibrium calculations, as if it
C were not even included in the data-file. The status "eliminated"
C allows one to selectively delete phases from equilibrium calculations,
C for instance to increase computational efficiency.

      END



