
C Changing the status of phase constituents

      SUBROUTINE TABLE
      IMPLICIT NONE

      INTEGER I, NOERR, NPCON
      DOUBLE PRECISION AMOUNT, FUG
      CHARACTER NAME*24, STATUS*24

C Retrieve and display the fugacities, equilibrium 
C amounts, and statuses of all phase constituents in the gas phase

C Get number of gas phase constituents
      CALL TQNOPC(1, NPCON, NOERR)

C Print table header
         WRITE(*,FMT='(A17,9X,A14,2X,A14,3X,A6)') 
     *     'Phase const. name',
     *     'amount/mol',
     *     'fugacity',
     *     'status'

      DO I=1, NPCON
C Get the phase constituent name
         CALL TQGNPC(1, I, NAME, NOERR)

C Get its equilibrium amount
         CALL TQGETR('A ',  1, I, AMOUNT, NOERR)

C Get its fugacity
         CALL TQGETR('AC ', 1, I, FUG, NOERR)

C Get its status
         CALL TQGSPC(1, I, STATUS, NOERR)
         WRITE(*,FMT='(A24,2X,G14.5,2X,G14.5,2X,A10)') 
     *        NAME, AMOUNT, FUG, STATUS
      ENDDO
      RETURN
      END

      PROGRAM CAF15
      IMPLICIT NONE
      
      INTEGER I, NOERR, NUMCON, NPCON
      INTEGER ISIO2, IC, ISI
      DOUBLE PRECISION VALS(2), STOI(3), WMASS

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading 
      CALL TQOPNA('cosi.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Unless a phase constituent is marked "dormant" or "eliminated" in the
C data-file itself, the status is by default always "entered". This
C means it is considered in the subsequent equilibrium calculation and
C is included in the mass balances.

C Set some conditions and perform a simple calculation to 
C demonstrate the effect of the different statuses

C Set temperature to 1900 K
      CALL TQSETC('T ', 0, 0, 1900.D0, NUMCON, NOERR)

C Set incoming amount to 1 mol SiO2(quartz) and 2 mol C
      CALL TQINP('SiO2(quartz)', ISIO2, NOERR)
      CALL TQSETC('IA ', ISIO2, 0, 1.D0, NUMCON, NOERR)

      CALL TQINP('C ', IC, NOERR)
      CALL TQSETC('IA ', IC, 0, 2.D0, NUMCON, NOERR)

C Calculate the equilibrium
      CALL TQCE(' ', 0, 0, VALS, NOERR)

C Print a table containing the phase constituent names, their
C equilibrium amounts, activities, and status
      CALL TABLE


C To demonstrate the effect of assigning a different status to phase
C constituents, set the status of all those phase constituents of the
C gas phase to "dormant" which contain Si.

C Get number of gas phase constituents
      CALL TQNOPC(1, NPCON, NOERR)

C Get the index number of system component Si
      CALL TQINSC("Si", ISI, NOERR)

C Use TQSTPC to determine for each phase constituent whether it
C contains Si, and set its status to "dormant" if it does
      DO I=1, NPCON
         CALL TQSTPC(1, I, STOI, WMASS, NOERR)
         IF (STOI(ISI) .GT. 0.D0) THEN
            CALL TQCSPC(1, I, 'DORMANT ', NOERR)
         ENDIF
      ENDDO

C Calculate the equilibrium and print the table again
      CALL TQCE(' ', 0, 0, VALS, NOERR)
      CALL TABLE


C As can be seen from the table above, all phase constituents of the gas
C phase that contain Si have been set to "dormant". ChemApp considers
C them in the activity calculation, but does not include them in the
C mass balances: their equilibrium amount is always zero. Since this
C forces ChemApp to keep the gas phase free of Si compounds, the 1 mol
C of Si input into the system can thus only be found in the condensed
C phases.

C Setting the status of these phase constituents to "eliminated" instead
C would cause ChemApp to exclude them completely from the equilibrium
C calculation.

      END

