
C Equilibrium calculations using streams

      PROGRAM CAF17
      IMPLICIT NONE
      
      INTEGER NOERR, ISIO2, IO2, ICO2, I, NPHASE
      DOUBLE PRECISION TPSOL(2), TPGAS(2), VALS(2), AMOUNT,
     * VALUE
      CHARACTER PNAME*24, SNAME*24

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading 
      CALL TQOPNA('cosi.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Two input streams will subsequently be defined: The first one,
C labelled 'STREAM_QUARTZ', will be solid SiO2(quartz) at 500 K, the
C second one, labelled 'STREAM_GAS', will consist of a gas phase at room
C temperature consisting of O2 and CO2. By using streams instead of
C global conditions we will be able to get information about the
C extensive property balance of this process.

C Define the first stream ('STREAM_QUARTZ')
      
C Use the array called TPSOL to pass the information on temperature and
C pressure (500 K and 1 bar) of the stream to TQSTTP
      TPSOL(1) = 500.D0
      TPSOL(2) = 1.D0
      CALL TQSTTP('STREAM_QUARTZ ', TPSOL, NOERR)

C Define the amounts of constituents of this stream (1 mol quartz)
      CALL TQINP('SiO2(quartz)', ISIO2, NOERR)
      CALL TQSTCA('STREAM_QUARTZ ', ISIO2, 0, 1.D0, NOERR)

C Define second stream ('STREAM_GAS')
      TPGAS(1) = 298.15D0
      TPGAS(2) = 1.D0
      CALL TQSTTP('STREAM_GAS ', TPGAS, NOERR)
      
C Define the amounts of constituents of this stream (0.5 mol O2/GAS/ and
C 0.5 mol CO2/GAS/)
      CALL TQINPC('O2 ', 1, IO2, NOERR)
      CALL TQSTCA('STREAM_GAS ', 1, IO2, 0.5D0, NOERR)

      CALL TQINPC('CO2 ', 1, ICO2, NOERR)
      CALL TQSTCA('STREAM_GAS ', 1, ICO2, 0.5D0, NOERR)

C Set the temperature at which the two streams should react
      CALL TQSTEC('T ', 0, 2500.D0, NOERR)

C Define a third stream for the sole purpose of demonstrating how to
C remove it subsequently
      CALL TQSTTP('DUMMY ', TPGAS, NOERR)
      CALL TQSTCA('DUMMY ', 1, 1, 1.D0, NOERR)

C Display what we have set so far
      CALL TQSHOW(NOERR)


C Remove the 'DUMMY' stream
      CALL TQSTRM('DUMMY ', NOERR)
      
C Call TQSHOW again to demonstrate that the last stream has been removed
      CALL TQSHOW(NOERR)


C Calculate the equilibrium
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


 50   FORMAT(1X,A,G12.5,A)

C Get the enthalpies of the two streams (note that as of version 3.2.0
C of ChemApp, TQSTXP can also be called _before_ the equilibrium
C calculation)
      SNAME = 'STREAM_QUARTZ'
      CALL TQSTXP(SNAME, 'H ', VALUE, NOERR)
      WRITE(*,FMT=50) 'Enthalpy of stream '// SNAME // ': ',
     *     VALUE, ' J'
      SNAME = 'STREAM_GAS'
      CALL TQSTXP(SNAME, 'H ', VALUE, NOERR)
      WRITE(*,FMT=50) 'Enthalpy of stream '// SNAME // ': ',
     *     VALUE, ' J'


C Get the list of stable phases
      WRITE(*,FMT='(A)') ' List of stable phases:'
      CALL TQNOP(NPHASE, NOERR)
      DO I=1, NPHASE
         CALL TQGETR('A ', I, 0, AMOUNT, NOERR)
         IF (AMOUNT .GT. 0) THEN
            CALL TQGNP(I, PNAME, NOERR)
            WRITE(*,FMT=50) 'Phase: ' // PNAME // 
     *           'amount: ', AMOUNT, ' mol' 
         ENDIF
      ENDDO


C Get the changes in the extensive property values in the current units
      WRITE(*,'(A)') ' Changes in extensive properties:'
      CALL TQGETR('CP ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta CP: ', VALUE, ' J/K'
      CALL TQGETR('H ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta H:  ', VALUE, ' J'
      CALL TQGETR('S ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta S:  ', VALUE, ' J/K'
      CALL TQGETR('G ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta G:  ', VALUE, ' J'
      CALL TQGETR('V ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta V:  ', VALUE, ' dm^3'


C To retrieve the *total* volume, use the option 'VT'
      CALL TQGETR('VT ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Total volume:  ', VALUE, ' dm^3'


      END

