
C Calculation of different phase equilibria 

C Note that this program contains target calculations,
C which cannot be performed with the 'light' version of ChemApp

      PROGRAM CAF18
      IMPLICIT NONE
      
      INTEGER NOERR, ISLITE, IO2, IC, IN2, ITEMP
      DOUBLE PRECISION VALS(2), TP(2), RESULT, TEMP

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading. In this case, a data-file containing the
C thermochemical data for the system carbon-nitrogen-oxygen is selected.
      CALL TQOPNA('cno.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Since we will perform an extensive property target calculation later,
C we will do the equilibrium calculations using streams. This way we can
C associate an initial state regarding temperature and pressure to the
C incoming substances, which will be taken into account when ChemApp
C calculates the extensive property (e.g. heat) balance.

C Since we are going to prefer degrees Celsius as temperature unit,
C we will change the unit now
      CALL TQCSU('Temperature ', 'C ', NOERR)

C Define a stream at room temperature and 1 bar
      TP(1) = 20.D0
      TP(2) = 1.D0
      CALL TQSTTP('inputs ', TP, NOERR)

C Set stream constituents: 0.5 mol of oxygen and 10 grams of carbon

C Get the index number of the phase constituent oxygen ('O2') in the gas
C phase. If a thermochemical data-file for ChemApp and ChemSage
C contains a gas phase, it is always the first one:
      CALL TQINPC('O2 ', 1, IO2, NOERR)

C Get the index number of the phase carbon ('C')
      CALL TQINP('C ', IC, NOERR)

C Set the incoming amount of oxygen in the stream (0.5 mol)
      CALL TQSTCA('inputs ', 1, IO2, .5D0, NOERR)

C Change the amount unit to gram and set the incoming amount of carbon
C in the stream (10 gram)
      CALL TQCSU('Amount ', 'gram ', NOERR)
      CALL TQSTCA('inputs ', IC, 0, 10.D0, NOERR)

C Set the temperature for the equilibrium calculation (1500 C)
      CALL TQSTEC('T ', 0, 1500.D0, NOERR)

C Calculate the equilibrium
      CALL TQCE(' ', 0, 0, VALS, NOERR)

C Alternatively, TQCEL can be used, which does not only calculate the
C equilibrium, but also writes a ChemSage result table to standard output
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


 50   FORMAT(1X,A,G12.5)

C Get the change in volume that takes place
      CALL TQGETR('V ', 0, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Change in volume/dm^3 is ', RESULT


C Get the enthalpy change for the process (since its sign is negative,
C energy is released
      CALL TQGETR('H ', 0, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Change in enthalpy/J is ', RESULT

      
C Get the activity of carbon, which, at 1500 C is far from being unity
      CALL TQGETR('AC ', IC, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Activity of carbon is ', RESULT


C Once an equilibrium has been calculated using TQCE/TQCEL, subsequent
C calculations can be performed using TQCEN/TQCENL, which speeds up the
C equilibrium calculation by taking results from the previous
C calculation as initial estimates

      DO ITEMP=1510, 1600, 10

         TEMP = ITEMP

         CALL TQSTEC('T ', 0, TEMP, NOERR)
         CALL TQCEN(' ', 0, 0, VALS, NOERR)
         CALL TQGETR('AC ', IC, 0, RESULT, NOERR)
         WRITE(*,*) 'Activity of carbon at T = ', 
     *        TEMP, ' C is ', RESULT

      ENDDO


         
C Similar to TQCEL, TQCENL also provides a ChemSage output table
      CALL TQCENL(' ', 0, 0, VALS, NOERR)

      

C Check if we are working with the 'light' version.
C If we do, omit the following target calculation(s).
      CALL TQLITE(ISLITE, NOERR)
      IF (ISLITE .EQ. 1) THEN
         WRITE(*,FMT='(3(1X,A,/))') 
     *     '*** Target calculations have been omitted here,',
     *     '*** since they are not possible with the ',
     *     '*** ''light'' version of ChemApp.'

      ELSE

C PERFORMING target calculations
C
C Note that target calculations are not possible with ChemApp "light",
C you need the regular version of ChemApp to run the examples below.
C
C Performing target calculations requires that ChemApp needs to be told
C 2 things: 
C - the target itself,
C - the target variable, i.e. which variable ChemApp is allowed to vary
C   to achieve the target.
C      
C First we will perform a formation phase target calculation, which is
C used to determine the conditions under which a phase becomes stable,
C using carbon as an example. As a target variable we will use the
C temperature. The result will be the temperature under which solid
C carbon will just become stable (smoke point).

C Define the target: phase C, amount of C = 0
      CALL TQSTEC('A ', IC, 0.D0, NOERR)

C Define the target variable, which is done during the call to TQCE.
C The value passed as VALS(1) will be taken as an initial estimate.
      VALS(1) = 500.D0
      CALL TQCE('T ', 0, 0, VALS, NOERR)

C Get the temperature, which ChemApp determined
      CALL TQGETR('T ', 0, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Calculated formation temperature of ' // 
     *     'carbon in degrees Celsius: ', RESULT


C Check whether ChemApp calculated correctly by verifying that the
C activity of carbon is unity.
      CALL TQGETR('AC ', IC, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Activity of carbon is ', RESULT


C The equilibrium amount of carbon should still be zero though, as we
C had ChemApp find the temperature where carbon _just_ gets stable
      CALL TQGETR('A ', IC, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Amount of carbon is ', RESULT


C It is also possible to specify a different amount than zero for the
C phase formation target calculation. To to this, pass a different value
C to TQSTEC when defining the target. As an example, calculate the
C temperature where 1 gram of carbon is stable
      CALL TQSTEC('A ', IC, 1.D0, NOERR)
      CALL TQCE('T ', 0, 0, VALS, NOERR)
      CALL TQGETR('T ', 0, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Calculated formation temperature for ' // 
     *     '1 gram of carbon in deg. Celsius:',
     *     RESULT

      
      ENDIF

C Before performing extensive property target calculations, TQREMC is
C used to remove all conditions and targets set previously.
      CALL TQREMC(-2, NOERR)
      
C Remove also the stream 'inputs'
      CALL TQSTRM('inputs ', NOERR)

C It is also useful to reset VALS, which now still contains a starting
C value for a temperature in VALS(1)
      VALS(1) = 0

C If it is also desired to reset all units to default units, which would
C in this case reset the amount to mol and the temperature unit to
C Kelvin, call TQREMC with a value of -1
      CALL TQREMC(-1, NOERR)

C Set up a new stream which contains 1 mol of pure oxygen, preheated to
C 400 K at 1 bar
      TP(1) = 400.0
      TP(2) = 1.0
      CALL TQSTTP('oxygen ', TP, NOERR)
      CALL TQSTCA('oxygen ', 1, IO2, 1.D0, NOERR)
      
C Another stream that contains 1 mol of solid carbon at room
C temperature and 1 bar
      TP(1) = 298.15
      TP(2) = 1.0
      CALL TQSTTP('carbon ', TP, NOERR)
      CALL TQSTCA('carbon ', IC, 0, 1.D0, NOERR)
      
C Calculate the equilibrium and retrieve the enthalpy released for this
C process
      CALL TQCE(' ', 0, 0, VALS, NOERR)
      CALL TQGETR('H ', 0, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Change in enthalpy/J is ', RESULT


C Check if we are working with the 'light' version.
C If we do, omit the following target calculation(s).
      CALL TQLITE(ISLITE, NOERR)
      IF (ISLITE .EQ. 1) THEN
         WRITE(*,FMT='(3(1X,A,/))') 
     *     '*** Target calculations have been omitted here,',
     *     '*** since they are not possible with the ',
     *     '*** ''light'' version of ChemApp.'

      ELSE

C Determine the adiabatic temperature for this process. This would be
C the temperature at which the above mentioned enthalpy is zero

C Define the target: enthalpy, value = 0
      CALL TQSTEC('H ', 0, 0.D0, NOERR)

C Define the target variable, which is done during the call to TQCE.
C The value passed as VALS(1) will be taken as an initial estimate.
      VALS(1) = 1000.D0
      CALL TQCE('T ', 0, 0, VALS, NOERR)

C Get the temperature ChemApp calculated (which is the adiabatic
C temperature in this example)
      CALL TQGETR('T ', 0, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Calculated adiabatic temperature/K for ' // 
     *     'this process: ', RESULT


C One can investigate how this temperature changes if one assumes a
C mixture that resembles air, instead of pure O2. Air has an N2/O2 ratio
C of about 4:1. Keeping 1 mol of oxygen, we thus have to add 4 mol of
C N2.
      CALL TQINPC('N2 ', 1, IN2, NOERR)
      CALL TQSTCA('oxygen ', 1, IN2, 4.D0, NOERR)
     
      CALL TQCE('T ', 0, 0, VALS, NOERR)

      CALL TQGETR('T ', 0, 0, RESULT, NOERR)
      WRITE(*,FMT=50) 'Adiabatic temperature/K for this process ' // 
     *     'with an N2/O2 mixture: ', RESULT


      ENDIF
      END

