
C Retrieving results from equilibrium calculations

      PROGRAM CAF19
      IMPLICIT NONE
      
      INTEGER NOERR, ISIO2, ICO2, INDEXP, NSC
      DOUBLE PRECISION TP(2), VALS(2), VALUE, VALUES(3)
      CHARACTER SCNAME*24

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading 
      CALL TQOPNA('cosi.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Define a sample input stream ('INPUTS')
      
C Use the array called TP to pass the information on temperature and
C pressure (298.15 K and 1 bar) of the stream to TQSTTP
      TP(1) = 298.15D0
      TP(2) = 1.D0
      CALL TQSTTP('INPUTS ', TP, NOERR)

C Define the amounts of constituents of this stream (1 mol quartz and
C 0.5 mol CO2)
      CALL TQINP('SiO2(quartz)', ISIO2, NOERR)
      CALL TQSTCA('INPUTS ', ISIO2, 0, 1.D0, NOERR)
      CALL TQINPC('CO2 ', 1, ICO2, NOERR)
      CALL TQSTCA('INPUTS ', 1, ICO2, 0.5D0, NOERR)

C Set the temperature at which the two streams should react
      CALL TQSTEC('T ', 0, 2500.D0, NOERR)

C Calculate the equilibrium
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


 50   FORMAT(1X,A,G12.5,A)

C A number of the results retrieved below can also be found in the
C ChemSage result table output above by TQCEL. If you do not want a
C ChemSage result table, for instance because your application program
C has a graphical user interface and cannot make use of the table, call
C TQCE instead to calculate the equilibrium.

C Get the pressure, which is 1 bar, since this is the default.
      CALL TQGETR('P ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Equilibrium pressure: ', VALUE, ' bar'

      
C Get the calculated _total_ equilibrium volume
      CALL TQGETR('VT ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Equilibrium volume (total): ', VALUE, ' dm^3'

      
C Get the calculated _change_ in volume. Note that options 'V' and 'VT'
C _only_ give different values if streams are used _and_ the whole
C system is selected, in which case 'V' retrieves the _change_ in volume
C between the reaction products and the stream constituents. In all
C other cases, both 'VT' and 'V' give the _total_ volume.
      CALL TQGETR('V ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Change in volume: ', VALUE, ' dm^3'

       
C Get the temperature, which is 2500 K since we set this particular
C condition for the equilibrium calculation
      CALL TQGETR('T ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Equilibrium temperature: ', VALUE, ' K'


C Get the equilibrium amount of liquid SiO2
      CALL TQINP('SiO2(liquid)', INDEXP, NOERR)
      CALL TQGETR('A ', INDEXP, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Equilibrium amount of SiO2(liquid): ', 
     *     VALUE, ' mol'


C Get the incoming amount of CO2/GAS/ that was input for the last
C equilibrium calculation
      CALL TQGETR('IA ', 1, ICO2, VALUE, NOERR)
      WRITE(*,FMT=50) 'Incoming amount of CO2/GAS/: ', 
     *     VALUE, ' mol'

      
C Get the chemical potential of SiO2(quartz)
      CALL TQGETR('MU ', 1, ISIO2, VALUE, NOERR)
      WRITE(*,FMT=50) 'Chemical potential of SiO2(quartz): ', 
     *     VALUE, ' J/mol'


C Get the activity of SiO2(quartz)
      CALL TQGETR('AC ', 1, ISIO2, VALUE, NOERR)
      WRITE(*,FMT=50) 'Activity of SiO2(quartz): ', 
     *     VALUE, ' '


C Get the activity of the first system component (carbon). Note that the
C activities of system components are given in absolute values, whereas
C relative values are obtained for phases and constituents.
      CALL TQGETR('AC ', 0, 1, VALUE, NOERR)
      WRITE(*,FMT=50) 'Activity of system component C: ', 
     *     VALUE, ' '

      
C Get the changes in the extensive property values between the reaction
C products and the reactants given as the stream constituents in the
C current units for the whole system (note that these are relative
C values)
      WRITE(*,'(A)') ' Changes in extensive properties:'
      CALL TQGETR('CP ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta CP: ', VALUE, ' J/K'
      CALL TQGETR('H ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta H:  ', VALUE, ' J'
      CALL TQGETR('S ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta S:  ', VALUE, ' J/K'
      CALL TQGETR('G ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta G:  ', VALUE, ' J'
      CALL TQGETR('V ', 0, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'Delta V:  ', VALUE, ' dm^3'


C Get the Gibbs energy value for CO2/GAS/
      CALL TQGETR('G ', 1, ICO2, VALUE, NOERR)
      WRITE(*,FMT=50) 'G(CO2/GAS/): ', VALUE, ' J'


C Get the Gibbs energy value for CO2/GAS/ per amount unit
      CALL TQGETR('GM ', 1, ICO2, VALUE, NOERR)
      WRITE(*,FMT=50) 'G_M(CO2/GAS/): ', VALUE, ' J/mol'


C Get the mole fraction of system component C in the system
      CALL TQGETR('X ', 0, 1, VALUE, NOERR)
      WRITE(*,FMT=50) 'Mole fraction of C in the system: ', 
     *     VALUE, ' '


C Get the mole fraction of system component C in the gas phase
      CALL TQGETR('XP ', 1, 1, VALUE, NOERR)
      WRITE(*,FMT=50) 'Mole fraction of C in the gas phase: ', 
     *     VALUE, ' '


C Now get the total amount of system component C in the gas phase
      CALL TQGETR('AP ', 1, 1, VALUE, NOERR) 
      WRITE(*,FMT=50) 'Amount of system component C' //
     *     ' in the GAS phase: ', VALUE, ' mol'


C Now get the total amount of all system components in the gas phase,
C having TQGETR return all three values in one array
      CALL TQGETR('AP ', 1, -1, VALUES, NOERR) 
      WRITE(*,'(A)') ' Amounts of all system components' //
     *     ' in the GAS phase:'

      DO NSC = 1, 3
         CALL TQGNSC(NSC, SCNAME, NOERR)
         WRITE(*,FMT=50) 'Amount of ' // SCNAME // 
     *        ' in the GAS phase: ', VALUES(NSC), ' mol'
      ENDDO




      END

