
C Calculate thermodynamic quantities

      PROGRAM CAF20
      IMPLICIT NONE
      
      INTEGER NOERR, NUMCON, ISIO2
      DOUBLE PRECISION VALUE, R, T

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading 
      CALL TQOPNA('cosi.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

 50   FORMAT(1X,A,G12.5,A)

C Defining the gas constant in J/mol.K
      R=8.31451

C Get thermochemical function values for SiO2(quartz) at 500 K
      T = 500.D0
      CALL TQSETC('T ', 0, 0, T, NUMCON, NOERR)
      CALL TQINP('SiO2(quartz)', ISIO2, NOERR)

C Note that when retrieving CP, H, S, or G, the values are returned
C dimensionless, which means they might have to be multiplied by
C R*T. Since the default amount unit is mol, results are returned for 1
C mol. Note also that care has to be taken if a temperature unit
C different from Kelvin has been used.
      CALL TQGDPC('CP ', ISIO2, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'CP of SiO2(quartz) at 500 K: ', 
     *     VALUE*R*T, ' J/mol.K'

      CALL TQGDPC('H ', ISIO2, 0, VALUE, NOERR)
      WRITE(*,FMT=50) 'H of SiO2(quartz) at 500 K: ', 
     *     VALUE*R*T, ' J/mol'


C The option 'T' serves a special purpose: If used, it returns the end
C temperature of the current interval of the CP or G function for the
C selected phase or phase constituent. Looking at the data-file cosi.dat
C (see Appendix ref(cosidat)), one sees from the entry for SiO2(quartz)
C that for a temperature of 500 K, the temperature range of the G
C function in question extends to 848 K.
      CALL TQGDPC('T ', ISIO2, 0, VALUE, NOERR)
      WRITE(*,FMT='(A)') ' For SiO2(quartz) and T = 500 K, the present'
      WRITE(*,FMT=50) 'temperature interval ends at: ', VALUE, ' K'


C Setting a different temperature for which a different G function
C interval applies
      T = 1000.D0
      CALL TQSETC('T ', 0, 0, T, NUMCON, NOERR)
      CALL TQGDPC('T ', ISIO2, 0, VALUE, NOERR)
      WRITE(*,FMT='(A)') ' For SiO2(quartz) and T = 1000 K, the present'
      WRITE(*,FMT=50) 'temperature interval ends at: ', VALUE, ' K'


C The option 'T' of TQGDPC is especially useful when the thermodynamic
C values are calculated as a function of temperature, and the ranges of
C the CP or G functions need to be known explicitly in order to locate
C transition temperatures between different modifications and determine
C the associated changes in the thermodynamic values.

      END




