

C Using streams (advanced topics)

      PROGRAM CAF201
      IMPLICIT NONE
      
      INTEGER NOERR, IBCC, IFE, IC, I, NPHASE,
     * numcon, nopc
      
      DOUBLE PRECISION VAL, VALS(2), TP(2)

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading
      CALL TQOPNA('fec.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Get the index number of the BCC phase. It is actually called 'BCC_A2',
C but since 'BCC' is a unique abbreviation, it will do.
      CALL TQINP('BCC ', IBCC, NOERR)

C Get the number of phases in the data-file
      CALL TQNOP(NPHASE, NOERR)

C Eliminate all phases except BCC from the calculation
      DO I=1, NPHASE
         IF (I .NE. IBCC) CALL TQCSP(i, 'eliminated', noerr)
      ENDDO

C Enter the equilibrium temperature. It does not really matter which
C temperature is entered for this intermediate equilibrium calculation
C using global conditions, since BCC is the only phase which has not
C been eliminated.
      CALL TQSETC('T ', 0, 0, 800.D0, NUMCON, NOERR)
 
C Get the index numbers of the system components Fe and C
      CALL TQINSC('Fe ', IFE, NOERR)
      CALL TQINSC('C ', IC, NOERR)

C Enter the incoming amounts. In our example, an alloy consisting of
C 99.999 mol-% Fe, rest C.
      CALL TQSETC('ia ', 0, IFE, 0.99999D0, NUMCON, NOERR)
      CALL TQSETC('ia ', 0, IC,  0.00001D0, NUMCON, NOERR)

C Calculate the equilibrium
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


C As can be seen from the above output table, the BCC phase is the
C only one which is stable, since all others have been eliminated. 

C Before the final calculation using streams is performed, remove all
C previously set conditions.
      CALL TQREMC(-2, NOERR)

C Remove all streams too, in case any have been defined previously.
      CALL TQSTRM(' ',NOERR)

C Include all phases in the equilibrium calculation again.
      DO I=1, NPHASE
         CALL TQCSP(i, 'entered', NOERR)
      ENDDO

C Define a stream called 'STREAM_BCC' at 300 K and 1 bar
      TP(1) = 300.D0
      TP(2) = 1.D0
      CALL TQSTTP('STREAM_BCC ', TP, NOERR)

C Get the number of phase constituents in the BCC phase
      CALL TQNOPC(IBCC, NOPC, NOERR)

C Now, retrieve all equilibrium amounts of the BCC phase constituents
C from the previous equilibrium calculation and input them as stream
C constituents in the BCC stream
      DO I=1, NOPC
         CALL TQGETR('a ', IBCC, I, VAL, NOERR)
         CALL TQSTCA('STREAM_BCC ', IBCC, I, VAL, NOERR)
      ENDDO
      
C Calculate the equilibrium
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


C As can be seen from the bottom of the above output table, the mole
C fractions of the system components in the BCC phase are, as intended,
C the same as in the first calculation. 

 10   CONTINUE
      
      END



