

C Using streams (advanced topics)

C Set the status of the phase INDEXP to 'entered', all other phases to
C 'eliminated'. If INDEXP is zero, set all phases and all their phase
C constituents (i.e. the whole system) to 'entered'.
      SUBROUTINE ENTERP(INDEXP)
      IMPLICIT NONE
      INTEGER INDEXP, NOP, NOPC, I, J, NOERR
      
      CALL TQNOP(NOP, NOERR)

      IF (INDEXP .EQ. 0) THEN
         DO I=1, NOP
            CALL TQCSP(I, 'ENTERED ', NOERR)
            CALL TQNOPC(I, NOPC, NOERR)
            DO J=1, NOPC
               CALL TQCSPC(I, J, 'ENTERED ', NOERR)
            ENDDO
         ENDDO
      ELSE
         DO I=1, NOP
            IF (I .EQ. INDEXP) THEN
               CALL TQCSP(I, 'ENTERED ', NOERR)
            ELSE
               CALL TQCSP(I, 'ELIMINATED ', NOERR)
            ENDIF
         ENDDO
      ENDIF

      RETURN
      END
      
C Set the status of phase constituent INDEXC of phase INDEX to
C 'entered', all other constituents of phase INDEXP to 'eliminated'. If
C INDEXC is zero, set all constituents of phase INDEXP to 'entered'.
      SUBROUTINE ENTERPC(INDEXP, INDEXC)
      IMPLICIT NONE
      INTEGER INDEXP, INDEXC, NOPC, I, NOERR
      
      CALL TQNOPC(INDEXP, NOPC, NOERR)

      IF (INDEXC .EQ. 0) THEN
         DO I=1, NOPC
            CALL TQCSPC(INDEXP, I, 'ENTERED ', NOERR)
         ENDDO
      ELSE
         DO I=1, NOPC
            IF (I .EQ. INDEXC) THEN
               CALL TQCSPC(INDEXP, I, 'ENTERED ', NOERR)
            ELSE
               CALL TQCSPC(INDEXP, I, 'ELIMINATED ', NOERR)
            ENDIF
         ENDDO
      ENDIF

      RETURN
      END
      

C Calculate the amounts of system components for a amount VAL of
C constituent INDEXC of phase INDEX. Add the values to the array of
C system components ASC and the array of total values of system
C components (TASC).
      SUBROUTINE CALCSC(NSC, INDEXP, INDEXC, VAL, ASC, TASC)
      IMPLICIT NONE
      INTEGER INDEXP, INDEXC, NSC, NOERR, I
      DOUBLE PRECISION VAL, STOI(30), ASC(30), TASC(30), WMASS

C These two calls are temporary, since this is not a general solution!
      CALL ENTERP(INDEXP)
      CALL ENTERPC(INDEXP, INDEXC)

      CALL TQSTPC(INDEXP, INDEXC, STOI, WMASS, NOERR)
      DO I=1, NSC
         ASC(I) = ASC(I) + STOI(I) * VAL
         TASC(I) = TASC(I) + STOI(I) * VAL
      ENDDO

      RETURN
      END

C Calculate an equilibrium using global conditions, using the system
C component amounts in ASC, as well as T and P in TP. The extensive
C properties are retrieved and added to the array holding the total
C amounts of the extensive property values (TEP)
      SUBROUTINE CALCGC(NSC, ASC, TP, TEP)
      IMPLICIT NONE
      INTEGER NSC, I, NUMCON, NOERR
      DOUBLE PRECISION ASC(30), TP(2), TEP(5), VALS(2), VAL

      CALL TQREMC(-2, NOERR)

      DO I=1, NSC
         CALL TQSETC('IA ', 0, I, ASC(I), NUMCON, NOERR)
      ENDDO

      CALL TQSETC('T ', 0, 0, TP(1), NUMCON, NOERR)
      CALL TQSETC('P ', 0, 0, TP(2), NUMCON, NOERR)

      CALL TQCEL(' ', 0, 0, VALS, NOERR)

      CALL TQGETR('CP ', 0, 0, VAL, NOERR)
      TEP(1) = TEP(1) + VAL
      CALL TQGETR('H ', 0, 0, VAL, NOERR)
      TEP(2) = TEP(2) + VAL
      CALL TQGETR('S ', 0, 0, VAL, NOERR)
      TEP(3) = TEP(3) + VAL
      CALL TQGETR('G ', 0, 0, VAL, NOERR)
      TEP(4) = TEP(4) + VAL
      CALL TQGETR('V ', 0, 0, VAL, NOERR)
      TEP(5) = TEP(5) + VAL

      RETURN
      END

C Calculate an equilibrium using global conditions, using the total
C amounts of system component amounts in TASC, as well as T and P in
C TP. TEP is used to calculate the resulting extensive property balance,
C the results are stored in EPB.
      SUBROUTINE CALCE(NSC, TASC, TP, TEP, EPB)
      IMPLICIT NONE
      INTEGER NSC, I, NUMCON, NOERR
      DOUBLE PRECISION TASC(30), TP(2), TEP(5), VALS(2), VAL, EPB(5)

      CALL TQREMC(-2, NOERR)
      
      DO I=1, NSC
         CALL TQSETC('IA ', 0, I, TASC(I), NUMCON, NOERR)
      ENDDO

      CALL TQSETC('T ', 0, 0, TP(1), NUMCON, NOERR)
      CALL TQSETC('P ', 0, 0, TP(2), NUMCON, NOERR)

      CALL TQCEL(' ', 0, 0, VALS, NOERR)

      CALL TQGETR('CP ', 0, 0, VAL, NOERR)
      EPB(1) = VAL - TEP(1)
      CALL TQGETR('H ', 0, 0, VAL, NOERR)
      EPB(2) = VAL - TEP(2)
      CALL TQGETR('S ', 0, 0, VAL, NOERR)
      EPB(3) = VAL - TEP(3)
      CALL TQGETR('G ', 0, 0, VAL, NOERR)
      EPB(4) = VAL - TEP(4)
      CALL TQGETR('V ', 0, 0, VAL, NOERR)
      EPB(5) = VAL - TEP(5)

      RETURN
      END

      PROGRAM CAF202
      IMPLICIT NONE
      
      INTEGER NOERR, IBCC, IFE, IC, I, NPHASE,
     * numcon, nopc
      INTEGER  IGAS, ICO2, ICO, IO2, IN2, NSC
      
      DOUBLE PRECISION VAL, VALS(2), TP(2), EPB(5), TEP(5),
     * ASC(30), TASC(30) 



C Initialise ChemApp 
      CALL TQINI(NOERR)

C Open data-file for reading
      CALL TQOPNA('cno.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Get index numbers of phases and phase constituents involved
      CALL TQINP('GAS ', IGAS, NOERR)
      CALL TQINPC('CO2 ', IGAS, ICO2, NOERR)
      CALL TQINPC('CO ', IGAS, ICO, NOERR)
      CALL TQINPC('O2 ', IGAS, IO2, NOERR)
      CALL TQINPC('N2 ', IGAS, IN2, NOERR)

C Change the temperature unit to degrees Celsius
      CALL TQCSU('Temperature ', 'C ', NOERR)

C Initialise the array of total extensive property values to zero
      DO I=1, 5 
         TEP(I) = 0.D0
      ENDDO

C Get the number of system components, which is mainly used in upper
C bounds of DO loops
      CALL TQNOSC(NSC, NOERR)
      
C Initialise the arrays holding the amounts of system components and the
C total amounts of system components
      DO I=1, NSC
         TASC(I) = 0.D0
         ASC(I) = 0.D0
      ENDDO

C Define 1st 'stream'
      CALL CALCSC(NSC, IGAS, ICO2, 0.5D0, ASC, TASC)
      TP(1) = 200.D0
      TP(2) = 1.D0
      CALL CALCGC(NSC, ASC, TP, TEP)


C Define 2nd 'stream'
      DO I=1, NSC
         ASC(I) = 0.D0
      ENDDO
      CALL CALCSC(NSC, IGAS, ICO, 2.5D0, ASC, TASC)
      CALL CALCGC(NSC, ASC, TP, TEP)


C Define 3rd 'stream'
      DO I=1, NSC
         ASC(I) = 0.D0
      ENDDO
      CALL CALCSC(NSC, IGAS, IO2, 0.5D0, ASC, TASC)
      TP(1) = 25.D0
      CALL CALCGC(NSC, ASC, TP, TEP)


C Define 4th 'stream'
      DO I=1, NSC
         ASC(I) = 0.D0
      ENDDO
      CALL CALCSC(NSC, IGAS, IN2, 2.0D0, ASC, TASC)
      CALL CALCGC(NSC, ASC, TP, TEP)

      
C Set the status of the whole system to 'entered'
      CALL ENTERP(0)

C Calculate the equilibrium
      TP(1) = 600.D0
      CALL CALCE(NSC, TASC, TP, TEP, EPB)


C Print the extensive property balance
      WRITE(*,*) 'Extensive properties balance:'
      WRITE(*,*) EPB



 10   CONTINUE
      
      END



