
C Change the limits of target variables

C Note that this program contains target calculations,
C which cannot be performed with the 'light' version of ChemApp

      PROGRAM CAF22
      IMPLICIT NONE
      
      INTEGER NOERR, ISLITE, NUMCON, IPB, ISN, ILIQ, IFCC
      DOUBLE PRECISION TEMP, ACT, VALS(2)

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Check if we are working with the 'light' version.
C If we do, omit the following target calculation(s).
      CALL TQLITE(ISLITE, NOERR)
      IF (ISLITE .EQ. 1) THEN
         WRITE(*,FMT='(3(1X,A,/))') 
     *     '*** Target calculations have been omitted here,',
     *     '*** since they are not possible with the ',
     *     '*** ''light'' version of ChemApp.'

      ELSE

C Open data-file for reading 
      CALL TQOPNA('pbsn.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

 50   FORMAT(1X,A,G12.5,A)
 51   FORMAT(1X,2(A,G12.5))

C For this example, let us assume one intends to find the solidus point
C for an Pb-Sn alloy consisting of 20 mol-% Sn and 80 mol-% Pb by
C performing a precipitation target calculation from the FCC
C phase. Figure ref(pbsn2_pd) shows the Pb-Sn phase diagram as
C calculated by ChemSage. The line marked "1" indicates the
C concentration that will be used for the precipitation target
C calculations below, using the temperature as the target variable.

C Get index numbers of system components
      CALL TQINSC('Pb ', IPB, NOERR)
      CALL TQINSC('Sn ', ISN, NOERR)

C Get index number of the FCC and liquid phases
      CALL TQINP('FCC ', IFCC, NOERR)         
      CALL TQINP('LIQ ', ILIQ, NOERR)

C Input 0.2 mol Sn and 0.8 mol Pb
      CALL TQSETC('IA ', 0, ISN, 0.2D0, NUMCON, NOERR)
      CALL TQSETC('IA ', 0, IPB, 0.8D0, NUMCON, NOERR)

C Define the precipitation phase target
      CALL TQSETC('A ', IFCC, 0, -1.D0, NUMCON, NOERR)

C Define the target variable (temperature) and perform the
C equilibrium calculation. The estimate for the target variable, passed
C via VALS(1), is set to 1000 K.
      VALS(1) = 1000.D0
      CALL TQCE('T ', 0, 0, VALS, NOERR) 

C Retrieve the calculated temperature
      CALL TQGETR('T ', 0, 0, TEMP, NOERR)
      WRITE(*,FMT=50) 'Calculated solidus temperature: ',
     *     TEMP, 'K'


C Looking at the Pb-Sn phase diagram, this temperature is certainly 
C wrong. This can be demonstrated by retrieving the activity of the
C liquid phase:
      CALL TQGETR('AC ', ILIQ, 0, ACT, NOERR)
      WRITE(*,FMT=51) 'Activity of liquid phase at ',TEMP, ' K: ',
     *     ACT


C Since the activity is less than unity, the liquid phase is not stable,
C thus this cannot be the solidus point. The reason is that ChemApp
C found the phase boundary between FCC and the two-phase region FCC+BCT,
C instead of that between FCC and FCC+Liquid. Note also that although the
C initial estimate for the target variable has been set to 1000 K, which
C is certainly closer to the solidus point than to the FCC/FCC+BCT phase
C boundary, this does not guarantee that of two possible solutions, the one
C closer to the initial estimate is found.

C To keep ChemApp from finding the unwanted phase boundary at lower
C temperatures, tell it to limit its search to temperatures above 450 K:
      CALL TQCLIM('TLOW ', 450.D0, NOERR)  

C Recalculate the equilibrium
      CALL TQCE('T ', 0, 0, VALS, NOERR) 

C Retrieve the calculated temperature
      CALL TQGETR('T ', 0, 0, TEMP, NOERR)
      WRITE(*,FMT=50) 'Calculated solidus temperature: ',
     *     TEMP, 'K'


C Check the activity of the liquid phase again to verify that it is 
C indeed stable: 
      CALL TQGETR('AC ', ILIQ, 0, ACT, NOERR)
      WRITE(*,FMT=51) 'Activity of liquid phase at ',TEMP, ' K: ',
     *     ACT


C Since the activity is unity, we know that the temperature we found is
C correct. 

      ENDIF
      END

