
C One-dimensional phase mapping calculation

C Note that this program contains target calculations,
C which cannot be performed with the 'light' version of ChemApp

      PROGRAM CAF24
      IMPLICIT NONE
      
      INTEGER NOERR, NUMCON, ISLITE, ISIO2, RESNO, ICONT
      DOUBLE PRECISION VALS(2), RESULT

      RESNO = 0

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading 
      CALL TQOPNA('cosi.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Check if we are working with the 'light' version.
C If we do, omit the following target calculation(s).
      CALL TQLITE(ISLITE, NOERR)
      IF (ISLITE .EQ. 1) THEN
         WRITE(*,FMT='(3(1X,A,/))') 
     *     '*** Target calculations have been omitted here,',
     *     '*** since they are not possible with the ',
     *     '*** ''light'' version of ChemApp.'

      ELSE

C Determine the index number for the phase SiO2(quartz)
      CALL TQINP('SiO2(quartz) ', ISIO2, NOERR)

C Enter one mol of SiO2
      CALL TQSETC('IA ', ISIO2, 0, 1.0D0, NUMCON, NOERR)


C The temperature search interval is supposed to range from 300 to 
C 3000 K:
      VALS(1) = 300.D0
      VALS(2) = 3000.D0

C First call to TQMAP, note the 'F' ('First') in the option parameter
      CALL TQMAP('TF ',0,0,VALS,ICONT,NOERR)
      
C The variable RESNO keeps track of the number of times we called TQMAP:
      RESNO = RESNO + 1

C Retrieve and print the temperature, which we know is VALS(1)
      CALL TQGETR('T ',0,0,RESULT,NOERR)

 100  FORMAT(A,F8.2,A)
      
      WRITE(*,100) '*** Lower interval boundary:', RESULT, ' K'

   30 CONTINUE 

C TQMAP is called again. Note the 'N' ('Next') in the option
C parameter. If we are at the first phase boundary (RESNO is still 2),
C we call TQMAPL for a change to produce a ChemSage output table...

      IF (RESNO .EQ. 2) THEN
         WRITE(*,FMT='(A)') 
     *   '*** ChemSage result table for the first phase boundary found:'
         CALL TQMAPL('TN ',0,0,VALS,ICONT,NOERR)
         WRITE(*,*)
C ...otherwise we just call TQMAP: 
      ELSE
         CALL TQMAP('TN ',0,0,VALS,ICONT,NOERR)
      ENDIF
      RESNO = RESNO +1

C Get the temperature...
      CALL TQGETR('T ',0,0,RESULT,NOERR)

C ... and print the entry for the result table. If we have called TQMAP
C twice already, we know that we have found a phase boundary. If not, we
C have retrieved the temperature value of the upper interval boundary
C VALS(2):
      IF (RESNO .GT. 2) THEN
         WRITE(*,100) '*** Phase boundary found at ', RESULT, ' K'
      ELSE
         WRITE(*,100) '*** Upper interval boundary: ', RESULT, ' K'
      ENDIF

C For as long as ICONT is positive, we need to make further calls of
C TQMAP
      IF (ICONT .GT. 0) GOTO 30


C With the above example the temperatures of all phase boundaries in a
C system which contains 1 mol of SiO2 have been calculated. Thus the
C phase boundaries determined reflect the stability ranges of the
C various modifications of SiO2. Also note again that the first two
C temperatures determined are _no_ phase boundaries, but the lower and
C upper limit of the search interval (VALS(1) and VALS(2)).
C
C For the first phase boundary found, TQMAPL is called instead of TQMAP,
C thus producing a ChemSage output table. From this ChemSage output
C table it can be seen that at the temperature calculated SiO2
C transforms from the quartz into the tridymite modification.
C
C Refer to the worked example 5 (One-dimensional phase mapping) for a
C further example. This also demonstrates how to obtain information on
C which phases are stable at each of the phase boundaries determined.

      ENDIF
      END



