
C Retrieving results for sublattice constituents

      PROGRAM CAF27
      IMPLICIT NONE
      
      INTEGER NOERR, ISLITE, NUMCON, I, INDEXP, INDEXL,
     *     INDEXC, NPHASE, NSL, NSLCON
      DOUBLE PRECISION VAL, VALS(2), RESULT
      CHARACTER PNAME*24, MNAME*24, CNAME*24


C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open the thermochemical data-file subl-ex.dat for reading. It contains
C an extract of the system Co-Cr-Fe: the phase SIGMA:30, which is
C modelled according to the sublattice formalism, and the BCC phase,
C described by a Redlich-Kister polynomial. Both phases are each
C included twice, to account for miscibility gaps.
      CALL TQOPNA('subl-ex.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Set the temperature to 1000 K
      CALL TQSETC('T ', 0, 0, 1000.D0, NUMCON, NOERR)

C Set the incoming amounts to 0.25 mol Co, 0.25 mol Cr, and 0.5 mol Fe
      CALL TQINSC('Co ', INDEXC, NOERR)
      CALL TQSETC('IA ', 0, INDEXC, 0.25D0, NUMCON, NOERR)

      CALL TQINSC('Cr ', INDEXC, NOERR)
      CALL TQSETC('IA ', 0, INDEXC, 0.25D0, NUMCON, NOERR)

      CALL TQINSC('Fe ', INDEXC, NOERR)
      CALL TQSETC('IA ', 0, INDEXC, 0.50D0, NUMCON, NOERR)


C Calculate the equilibrium
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


C As can be seen in the output above, the standard ChemSage result table
C obtained with TQCEL contains a section with details on the stable
C sublattice phases. The following code shows how to retrieve similar
C information with ChemApp.

C Loop over all phases, and if a phase is stable and a sublattice phase,
C print information about the sublattices
      CALL TQNOP(NPHASE, NOERR)
      DO INDEXP=1, NPHASE

C Check if the phase is stable, otherwise we don't need to consider it
         CALL TQGETR('A ', INDEXP, 0, VAL, NOERR)
         IF (VAL .GT. 0.D0) THEN

C Get the name and model of the phase
            CALL TQGNP(INDEXP, PNAME, NOERR)
            CALL TQMODL(INDEXP, MNAME, NOERR)

C If the model name of the phase starts with 'SUB' we have a sublattice
C phase
            IF (INDEX(MNAME,'SUB') .GT. 0) then
        
C Print a header similar to the one in the ChemSage output table
               WRITE(*,*) 'Mole fraction of the sublattice ' //
     *              'constituents in ', PNAME
    
C Get the number of sublattices
               CALL TQNOSL(INDEXP, NSL, NOERR)
            
C Loop over all sublattices
               DO INDEXL=1, NSL

                  write(*,*) 'Sublattice ',INDEXL,' :'
C Get the number of sublattice constituents
                  CALL TQNOLC(INDEXP, INDEXL, NSLCON, NOERR)

                  DO INDEXC=1, NSLCON
C Get the name of each sublattice constituent...
                     CALL TQGNLC(INDEXP, INDEXL, INDEXC, CNAME, NOERR)

C ... and its mole fraction in the sublattice
                     CALL TQGTLC(INDEXP, INDEXL, INDEXC, VAL, NOERR)
                  
                     WRITE(*,*) CNAME, VAL

                  ENDDO
               ENDDO
            ENDIF
         ENDIF
      ENDDO


      END











